from __future__ import division

import math
from itertools import chain, izip, permutations
from random import Random

SIZE = 2**10

edges2 = list(
    set(
        chain(
            permutations((0, 1, 1), 3),
            permutations((0, 1, -1), 3),
            permutations((0, -1, -1), 3),
        )
    )
)
edges2.sort()

edges3 = list(
    set(
        chain(
            permutations((0, 1, 1, 1), 4),
            permutations((0, 1, 1, -1), 4),
            permutations((0, 1, -1, -1), 4),
            permutations((0, -1, -1, -1), 4),
        )
    )
)
edges3.sort()

def dot2(u, v):
    """
    Dot product of two 2-dimensional vectors.
    """
    return u[0] * v[0] + u[1] * v[1]

def dot3(u, v):
    """
    Dot product of two 3-dimensional vectors.
    """
    return u[0] * v[0] + u[1] * v[1] + u[2] * v[2]

def reseed(seed):
    """
    Reseed the simplex gradient field.
    """

    if seed in fields:
        return

    p = range(SIZE)
    r = Random()
    r.seed(seed)
    r.shuffle(p)
    p *= 2
    fields[seed] = p

def set_seed(seed):
    """
    Set the current seed.
    """

    global current_seed

    reseed(seed)

    current_seed = seed

fields = dict()

current_seed = None

f2 = 0.5 * (math.sqrt(3) - 1)
g2 = (3 - math.sqrt(3)) / 6

def simplex2(x, y):
    """
    Generate simplex noise at the given coordinates.

    This particular implementation has very high chaotic features at normal
    resolution; zooming in by a factor of 16x to 256x is going to yield more
    pleasing results for most applications.

    The gradient field must be seeded prior to calling this function; call
    ``reseed()`` first.

    :param int x: X coordinate
    :param int y: Y coordinate

    :returns: simplex noise
    :raises Exception: the gradient field is not seeded
    """

    if current_seed is None:
        raise Exception("The gradient field is unseeded!")

    p = fields[current_seed]

    # Set up our scalers and arrays.
    coords = [None] * 3
    gradients = [None] * 3

    s = (x + y) * f2
    i = math.floor(x + s)
    j = math.floor(y + s)
    t = (i + j) * g2
    x -= i - t
    y -= j - t

    # Clamp to the size of the simplex array.
    i = int(i) % SIZE
    j = int(j) % SIZE

    # Look up coordinates and gradients for each contributing point in the
    # simplex space.
    coords[0] = x, y
    gradients[0] = p[i + p[j]]
    if x > y:
        coords[1] = x - 1 + g2, y     + g2
        gradients[1] = p[i + 1 + p[j    ]]
    else:
        coords[1] = x     + g2, y - 1 + g2
        gradients[1] = p[i     + p[j + 1]]
    coords[2] = x - 1 + 2 * g2, y - 1 + 2 * g2
    gradients[2] = p[i + 1 + p[j + 1]]

    # Do our summation.
    n = 0
    for coord, gradient in izip(coords, gradients):
        t = 0.5 - coord[0] * coord[0] - coord[1] * coord[1]
        if t > 0:
            n += t**4 * dot2(edges2[gradient % 12], coord)

    # Where's this scaling factor come from?
    return n * 70

def simplex3(x, y, z):
    """
    Generate simplex noise at the given coordinates.

    This is a 3-dimensional flavor of ``simplex2()``; all of the same caveats
    apply.

    The gradient field must be seeded prior to calling this function; call
    ``reseed()`` first.

    :param int x: X coordinate
    :param int y: Y coordinate
    :param int z: Z coordinate

    :returns: simplex noise
    :raises Exception: the gradient field is not seeded or you broke the
                       function somehow
    """

    if current_seed is None:
        raise Exception("The gradient field is unseeded!")

    p = fields[current_seed]

    f = 1 / 3
    g = 1 / 6
    coords = [None] * 4
    gradients = [None] * 4

    s = (x + y + z) * f
    i = math.floor(x + s)
    j = math.floor(y + s)
    k = math.floor(z + s)
    t = (i + j + k) * g
    x -= i - t
    y -= j - t
    z -= k - t

    i = int(i) % SIZE
    j = int(j) % SIZE
    k = int(k) % SIZE

    # Do the coord and gradient lookups. Unrolled for speed and clarity.
    # These should be + 2 * g, but instead we do + f because we already have
    # it calculated. (2g == 2/6 == 1/3 == f)
    coords[0] = x, y, z
    gradients[0] = p[i + p[j + p[k]]]
    if x >= y >= z:
        coords[1] = x - 1 + g, y     + g, z     + g
        coords[2] = x - 1 + f, y - 1 + f, z     + f

        gradients[1] = p[i + 1 + p[j     + p[k    ]]]
        gradients[2] = p[i + 1 + p[j + 1 + p[k    ]]]
    elif x >= z >= y:
        coords[1] = x - 1 + g, y     + g, z     + g
        coords[2] = x - 1 + f, y     + f, z - 1 + f

        gradients[1] = p[i + 1 + p[j     + p[k    ]]]
        gradients[2] = p[i + 1 + p[j     + p[k + 1]]]
    elif z >= x >= y:
        coords[1] = x     + g, y     + g, z - 1 + g
        coords[2] = x - 1 + f, y     + f, z - 1 + f

        gradients[1] = p[i     + p[j     + p[k + 1]]]
        gradients[2] = p[i + 1 + p[j     + p[k + 1]]]
    elif z >= y >= x:
        coords[1] = x     + g, y     + g, z - 1 + g
        coords[2] = x     + f, y - 1 + f, z - 1 + f

        gradients[1] = p[i     + p[j     + p[k + 1]]]
        gradients[2] = p[i     + p[j + 1 + p[k + 1]]]
    elif y >= z >= x:
        coords[1] = x     + g, y - 1 + g, z     + g
        coords[2] = x     + f, y - 1 + f, z - 1 + f

        gradients[1] = p[i     + p[j + 1 + p[k    ]]]
        gradients[2] = p[i     + p[j + 1 + p[k + 1]]]
    elif y >= x >= z:
        coords[1] = x     + g, y - 1 + g, z     + g
        coords[2] = x - 1 + f, y - 1 + f, z     + f

        gradients[1] = p[i     + p[j + 1 + p[k    ]]]
        gradients[2] = p[i + 1 + p[j + 1 + p[k    ]]]
    else:
        raise Exception("You broke maths. Good work.")

    coords[3] = x - 1 + 0.5, y - 1 + 0.5, z - 1 + 0.5
    gradients[3] = p[i + 1 + p[j + 1 + p[k + 1]]]

    n = 0
    for coord, gradient in izip(coords, gradients):
        t = (0.6 - coord[0] * coord[0] - coord[1] * coord[1] - coord[2] *
            coord[2])
        if t > 0:
            n += t**4 * dot3(edges2[gradient % 12], coord)

    # Where's this scaling factor come from?
    return n * 32

def simplex(*args):
    if len(args) == 2:
        return simplex2(*args)
    if len(args) == 3:
        return simplex3(*args)
    else:
        raise Exception("Don't know how to do %dD noise!" % len(args))

def octaves2(x, y, count):
    """
    Generate fractal octaves of noise.

    Summing increasingly scaled amounts of noise with itself creates fractal
    clouds of noise.

    :param int x: X coordinate
    :param int y: Y coordinate
    :param int count: number of octaves

    :returns: Scaled fractal noise
    """

    sigma = 0
    divisor = 1
    while count:
        sigma += simplex2(x * divisor, y * divisor) / divisor
        divisor *= 2
        count -= 1
    return sigma

def octaves3(x, y, z, count):
    """
    Generate fractal octaves of noise.

    :param int x: X coordinate
    :param int y: Y coordinate
    :param int z: Z coordinate
    :param int count: number of octaves

    :returns: Scaled fractal noise
    """

    sigma = 0
    divisor = 1
    while count:
        sigma += simplex3(x * divisor, y * divisor, z * divisor) / divisor
        divisor *= 2
        count -= 1
    return sigma

def offset2(x, y, xoffset, yoffset, octaves=1):
    """
    Generate an offset noise difference field.

    :param int x: X coordinate
    :param int y: Y coordinate
    :param int xoffset: X offset
    :param int yoffset: Y offset

    :returns: Difference of noises
    """

    return (octaves2(x, y, octaves) -
        octaves2(x + xoffset, y + yoffset, octaves) + 1) * 0.5



def gen_line_simple(point1, point2):
    """
    Generalization of Bresenham's line algorithm in 3d.

    It yields blocks coordinates along a line that is at all times only one
    block in width.

    It is a simplified version that does nothing fancy with integers or such:
    all computation is done on float (maybe it should be changed?)
    """

    tx, ty, tz = point1.x, point1.y, point1.z # t is for temporary
    rx, ry, rz = int (tx), int (ty), int (tz) # r is for rounded
    ox, oy, oz = point2.x, point2.y, point2.z # o is for objective

    dx = ox - tx
    dy = oy - ty
    dz = oz - tz

    largest = max (abs (dx), abs (dy), abs (dz)) * 1. # We want a float
    dx, dy, dz = dx / largest, dy / largest, dz / largest # We make a vector which maximum value is 1.0

    while abs (ox - tx) > 1 or abs (oy - ty) > 1 or abs (oz - tz) > 1:
        tx += dx
        ty += dy
        tz += dz
        if (ty < 0 and dy < 0) or (ty >= 127 and dy > 0):
            break
        rx, ry, rz = int (tx), int (ty), int (tz)
        yield rx, ry, rz

def gen_line_covered(point1, point2):
    """
    This is Bresenham's algorithm with a little twist: *all* the blocks that
    intersect with the line are yielded.
    """

    tx, ty, tz = point1.x, point1.y, point1.z # t is for temporary
    rx, ry, rz = int (tx), int (ty), int (tz) # r is for rounded
    ox, oy, oz = point2.x, point2.y, point2.z # o is for objective

    dx = ox - tx
    dy = oy - ty
    dz = oz - tz

    largest = max (abs (dx), abs (dy), abs (dz)) * 1. # We want a float
    dx, dy, dz = dx / largest, dy / largest, dz / largest # We make a vector which maximum value is 1.0
    adx, ady, adz = abs (dx), abs (dy), abs (dz)

    px, py, pz = rx, ry, rz
    while abs (ox - tx) > 1 or abs (oy - ty) > 1 or abs (oz - tz) > 1:
        tx += dx
        ty += dy
        tz += dz
        if (ty < 0 and dy < 0) or (ty >= 127 and dy > 0):
            break
        rx, ry, rz = int (tx), int (ty), int (tz)

        yield rx, ry, rz

        # Send blocks that are in fact intersected by the line
        # but that bresenham skipped.
        if rx != px and adx != 1:
            yield px, ry, rz

            if ry != py and ady != 1:
                yield px, py, rz

            if rz != pz and adz != 1:
                yield px, ry, pz

        if ry != py and ady != 1:
            yield rx, py, rz

            if rz != pz and adz != 1:
                yield rx, py, pz

        if rz != pz and adz != 1:
            yield rx, ry, pz

        px, py, pz = rx, ry, rz



# -*- test-case-name: twisted.python.test.test_components -*-
# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
# See LICENSE for details.


"""
Component architecture for Twisted, based on Zope3 components.

Using the Zope3 API directly is strongly recommended. Everything
you need is in the top-level of the zope.interface package, e.g.::

   from zope.interface import Interface, implements

   class IFoo(Interface):
       pass

   class Foo:
       implements(IFoo)

   print IFoo.implementedBy(Foo) # True
   print IFoo.providedBy(Foo()) # True

L{twisted.python.components.registerAdapter} from this module may be used to
add to Twisted's global adapter registry. 

L{twisted.python.components.proxyForInterface} is a factory for classes
which allow access to only the parts of another class defined by a specified
interface.
"""

# zope3 imports
from zope.interface import interface, declarations
from zope.interface.adapter import AdapterRegistry


# The global adapter registry of the modules package
globalRegistry = AdapterRegistry()

# Attribute that registerAdapter looks at. Is this supposed to be public?
ALLOW_DUPLICATES = 0

# Define a function to find the registered adapter factory, using either a
# version of Zope Interface which has the `registered' method or an older
# version which does not.
if getattr(AdapterRegistry, 'registered', None) is None:
    def _registered(registry, required, provided):
        """
        Return the adapter factory for the given parameters in the given
        registry, or None if there is not one.
        """
        return registry.get(required).selfImplied.get(provided, {}).get('')
else:
    def _registered(registry, required, provided):
        """
        Return the adapter factory for the given parameters in the given
        registry, or None if there is not one.
        """
        return registry.registered([required], provided)


def registerAdapter(adapterFactory, origInterface, *interfaceClasses):
    """Register an adapter class.

    An adapter class is expected to implement the given interface, by
    adapting instances implementing 'origInterface'. An adapter class's
    __init__ method should accept one parameter, an instance implementing
    'origInterface'.
    """
    self = globalRegistry
    assert interfaceClasses, "You need to pass an Interface"
    global ALLOW_DUPLICATES

    # deal with class->interface adapters:
    if not isinstance(origInterface, interface.InterfaceClass):
        origInterface = declarations.implementedBy(origInterface)

    for interfaceClass in interfaceClasses:
        factory = _registered(self, origInterface, interfaceClass)
        if factory is not None and not ALLOW_DUPLICATES:
            raise ValueError("an adapter (%s) was already registered." % (factory, ))
    for interfaceClass in interfaceClasses:
        self.register([origInterface], interfaceClass, '', adapterFactory)


def getAdapterFactory(fromInterface, toInterface, default):
    """Return registered adapter for a given class and interface.

    Note that is tied to the *Twisted* global registry, and will
    thus not find adapters registered elsewhere.
    """
    self = globalRegistry
    if not isinstance(fromInterface, interface.InterfaceClass):
        fromInterface = declarations.implementedBy(fromInterface)
    factory = self.lookup1(fromInterface, toInterface)
    if factory is None:
        factory = default
    return factory


# add global adapter lookup hook for our newly created registry
def _hook(iface, ob, lookup=globalRegistry.lookup1):
    factory = lookup(declarations.providedBy(ob), iface)
    if factory is None:
        return None
    else:
        return factory(ob)
interface.adapter_hooks.append(_hook)


def getRegistry():
    """Returns the Twisted global
    C{zope.interface.adapter.AdapterRegistry} instance.
    """
    return globalRegistry


__all__ = [
    # Sticking around:
    "registerAdapter", "getAdapterFactory",
    "getRegistry",
]



# Copyright (c) 2006-2010 Twisted Matrix Laboratories.
# See LICENSE for details.

"""
Tests for L{modules}, abstract access to imported or importable objects.
"""

import sys
import itertools
import zipfile
import compileall
try:
    import ast
except ImportError:
    ast = None

# Required for addCleanup and mktemp support, at least.
from twisted.trial.unittest import TestCase

# XXX This suggests that twisted.python.reflect should be the next (or previous)
# thing to release independently.  We could get it from filepath here, but we
# depend on Twisted for TestCase anyway so there's scarcely a point.
from twisted.python.reflect import namedAny

from exocet._filepath import FilePath
# XXX Check out zipit and see if we should copy it or what.
from exocet.test.test_paths import zipit

import exocet._modules as modules
from exocet._modules import _isPythonIdentifier, _ImportExportFinder

class PySpaceTestCase(TestCase):

    def findByIteration(self, modname, where=modules, importPackages=False):
        """
        You don't ever actually want to do this, so it's not in the public API, but
        sometimes we want to compare the result of an iterative call with a
        lookup call and make sure they're the same for test purposes.
        """
        for modinfo in where.walkModules(importPackages=importPackages):
            if modinfo.name == modname:
                return modinfo
        self.fail("Unable to find module %r through iteration." % (modname,))


    def replaceSysPath(self, sysPath):
        """
        Replace sys.path, for the duration of the test, with the given value.
        """
        originalSysPath = sys.path[:]
        def cleanUpSysPath():
            sys.path[:] = originalSysPath
        self.addCleanup(cleanUpSysPath)
        sys.path[:] = sysPath


    def replaceSysModules(self, sysModules):
        """
        Replace sys.modules, for the duration of the test, with the given value.
        """
        originalSysModules = sys.modules.copy()
        def cleanUpSysModules():
            sys.modules.clear()
            sys.modules.update(originalSysModules)
        self.addCleanup(cleanUpSysModules)
        sys.modules.clear()
        sys.modules.update(sysModules)


    def pathEntryWithOnePackage(self, pkgname="test_package"):
        """
        Generate a L{FilePath} with one package, named C{pkgname}, on it, and
        return the L{FilePath} of the path entry.
        """
        entry = FilePath(self.mktemp())
        pkg = entry.child("test_package")
        pkg.makedirs()
        pkg.child("__init__.py").setContent("")
        return entry



class BasicTests(PySpaceTestCase):

    def test_unimportablePackageGetItem(self):
        """
        If a package has been explicitly forbidden from importing by setting a
        C{None} key in sys.modules under its name,
        L{modules.PythonPath.__getitem__} should still be able to retrieve an
        unloaded L{modules.PythonModule} for that package.
        """
        shouldNotLoad = []
        path = modules.PythonPath(sysPath=[self.pathEntryWithOnePackage().path],
                                  moduleLoader=shouldNotLoad.append,
                                  importerCache={},
                                  sysPathHooks={},
                                  moduleDict={'test_package': None})
        self.assertEquals(shouldNotLoad, [])
        self.assertEquals(path['test_package'].isLoaded(), False)


    def test_unimportablePackageWalkModules(self):
        """
        If a package has been explicitly forbidden from importing by setting a
        C{None} key in sys.modules under its name, L{modules.walkModules} should
        still be able to retrieve an unloaded L{modules.PythonModule} for that
        package.
        """
        existentPath = self.pathEntryWithOnePackage()
        self.replaceSysPath([existentPath.path])
        self.replaceSysModules({"test_package": None})

        walked = list(modules.walkModules())
        self.assertEquals([m.name for m in walked],
                          ["test_package"])
        self.assertEquals(walked[0].isLoaded(), False)


    def test_nonexistentPaths(self):
        """
        Verify that L{modules.walkModules} ignores entries in sys.path which
        do not exist in the filesystem.
        """
        existentPath = self.pathEntryWithOnePackage()

        nonexistentPath = FilePath(self.mktemp())
        self.failIf(nonexistentPath.exists())

        self.replaceSysPath([existentPath.path])

        expected = [modules.getModule("test_package")]

        beforeModules = list(modules.walkModules())
        sys.path.append(nonexistentPath.path)
        afterModules = list(modules.walkModules())

        self.assertEquals(beforeModules, expected)
        self.assertEquals(afterModules, expected)


    def test_nonDirectoryPaths(self):
        """
        Verify that L{modules.walkModules} ignores entries in sys.path which
        refer to regular files in the filesystem.
        """
        existentPath = self.pathEntryWithOnePackage()

        nonDirectoryPath = FilePath(self.mktemp())
        self.failIf(nonDirectoryPath.exists())
        nonDirectoryPath.setContent("zip file or whatever\n")

        self.replaceSysPath([existentPath.path])

        beforeModules = list(modules.walkModules())
        sys.path.append(nonDirectoryPath.path)
        afterModules = list(modules.walkModules())

        self.assertEquals(beforeModules, afterModules)


    def test_modulesShowsUp(self):
        """
        Scrounge around in the top-level module namespace and make sure that
        L{modules} shows up, and that the module thusly obtained is the same as
        the module that we find when we look for it explicitly by name.
        """
        self.assertEquals(modules.getModule('exocet._modules'),
                          self.findByIteration("exocet._modules"))


    def test_lookForSource(self):
        """
        If a .py file exists for a loaded module, use it as the module path. If not, use __file__.
        """
        import pickle, datetime
        self.assertTrue(modules.getModule('pickle').filePath.path.endswith('.py'))
        self.assertFalse(modules.getModule('datetime').filePath.path.endswith('.py'))


    def test_dottedNames(self):
        """
        Verify that the walkModules APIs will give us back subpackages, not just
        subpackages.

        XXX Docstring makes no sense.
        """
        self.assertEquals(
            modules.getModule('exocet.test'),
            self.findByIteration("exocet.test",
                                 where=modules.getModule('exocet')))


    def test_onlyTopModules(self):
        """
        Verify that the iterModules API will only return top-level modules and
        packages, not submodules or subpackages.
        """
        for module in modules.iterModules():
            self.failIf(
                '.' in module.name,
                "no nested modules should be returned from iterModules: %r"
                % (module.filePath))


    def test_loadPackagesAndModules(self):
        """
        Verify that we can locate and load packages, modules, submodules, and
        subpackages.
        """
        for n in ['os',
                  'exocet',
                  'exocet.test',
                  'exocet.test.test_modules']:
            m = namedAny(n)
            self.failUnlessIdentical(
                modules.getModule(n).load(),
                m)
            self.failUnlessIdentical(
                self.findByIteration(n).load(),
                m)


    def test_pathEntriesOnPath(self):
        """
        Verify that path entries discovered via module loading are, in fact, on
        sys.path somewhere.
        """
        for n in ['os',
                  'exocet',
                  'exocet.test',
                  'exocet.test.test_modules']:
            self.failUnlessIn(
                modules.getModule(n).pathEntry.filePath.path,
                sys.path)


    def test_alwaysPreferPy(self):
        """
        Verify that .py files will always be preferred to .pyc files, regardless of
        directory listing order.
        """
        mypath = FilePath(self.mktemp())
        mypath.createDirectory()
        pp = modules.PythonPath(sysPath=[mypath.path])
        originalSmartPath = pp._smartPath
        def _evilSmartPath(pathName):
            o = originalSmartPath(pathName)
            originalChildren = o.children
            def evilChildren():
                # normally this order is random; let's make sure it always
                # comes up .pyc-first.
                x = originalChildren()
                x.sort()
                x.reverse()
                return x
            o.children = evilChildren
            return o
        mypath.child("abcd.py").setContent('\n')
        compileall.compile_dir(mypath.path, quiet=True)
        # sanity check
        self.assertEquals(len(mypath.children()), 2)
        pp._smartPath = _evilSmartPath
        self.assertEquals(pp['abcd'].filePath,
                          mypath.child('abcd.py'))


    def test_packageMissingPath(self):
        """
        A package can delete its __path__ for some reasons,
        C{modules.PythonPath} should be able to deal with it.
        """
        mypath = FilePath(self.mktemp())
        mypath.createDirectory()
        pp = modules.PythonPath(sysPath=[mypath.path])
        subpath = mypath.child("abcd")
        subpath.createDirectory()
        subpath.child("__init__.py").setContent('del __path__\n')
        sys.path.append(mypath.path)
        import abcd
        try:
            l = list(pp.walkModules())
            self.assertEquals(len(l), 1)
            self.assertEquals(l[0].name, 'abcd')
        finally:
            del abcd
            del sys.modules['abcd']
            sys.path.remove(mypath.path)


sampleModuleContents = """
import sys, os
from twisted.python import reflect
import twisted.python.filepath
from twisted.python.components import registerAdapter

foo = 123
def doFoo():
  import datetime
class Foo:
  x = 0
"""

sampleModuleWithExportsContents = """
import sys, os, datetime
from twisted.python import reflect
import twisted.python.filepath
from twisted.python.components import registerAdapter

foo = 123
baz = 456
__all__ = ['foo']
"""

sampleModuleWithExportedImportsContents = """
import sys, os, datetime
from twisted.python import reflect
import twisted.python.filepath
from twisted.python.components import registerAdapter

foo = 123
baz = 456
__all__ = ['foo', 'registerAdapter', 'reflect']
"""



class PathModificationTest(PySpaceTestCase):
    """
    These tests share setup/cleanup behavior of creating a dummy package and
    stuffing some code in it.
    """

    _serialnum = itertools.count().next # used to generate serial numbers for
                                        # package names.

    def setUp(self):
        self.pathExtensionName = self.mktemp()
        self.pathExtension = FilePath(self.pathExtensionName)
        self.pathExtension.createDirectory()
        self.packageName = "pyspacetests%d" % (self._serialnum(),)
        self.packagePath = self.pathExtension.child(self.packageName)
        self.packagePath.createDirectory()
        self.packagePath.child("__init__.py").setContent("")
        self.packagePath.child("a.py").setContent(sampleModuleContents)
        self.packagePath.child("b.py").setContent(sampleModuleWithExportsContents)
        self.packagePath.child("c__init__.py").setContent("")
        self.packagePath.child("d.py").setContent(sampleModuleWithExportedImportsContents)

        self.pathSetUp = False


    def _setupSysPath(self):
        assert not self.pathSetUp
        self.pathSetUp = True
        sys.path.append(self.pathExtensionName)


    def _underUnderPathTest(self, doImport=True):
        moddir2 = self.mktemp()
        fpmd = FilePath(moddir2)
        fpmd.createDirectory()
        fpmd.child("foozle.py").setContent("x = 123\n")
        self.packagePath.child("__init__.py").setContent(
            "__path__.append(%r)\n" % (moddir2,))
        # Cut here
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName)
        self.assertEquals(
            self.findByIteration(self.packageName+".foozle", modinfo,
                                 importPackages=doImport),
            modinfo['foozle'])
        self.assertEquals(modinfo['foozle'].load().x, 123)


    def test_underUnderPathAlreadyImported(self):
        """
        Verify that iterModules will honor the __path__ of already-loaded packages.
        """
        self._underUnderPathTest()


    def test_underUnderPathNotAlreadyImported(self):
        """
        Verify that iterModules will honor the __path__ of already-loaded packages.
        """
        self._underUnderPathTest(False)


    test_underUnderPathNotAlreadyImported.todo = (
        "This may be impossible but it sure would be nice.")


    def _listModules(self):
        pkginfo = modules.getModule(self.packageName)
        nfni = [modinfo.name.split(".")[-1] for modinfo in
                pkginfo.iterModules()]
        nfni.sort()
        self.failUnlessEqual(nfni, ['a', 'b', 'c__init__', 'd'])


    def test_listingModules(self):
        """
        Make sure the module list comes back as we expect from iterModules on a
        package, whether zipped or not.
        """
        self._setupSysPath()
        self._listModules()


    def test_listingModulesAlreadyImported(self):
        """
        Make sure the module list comes back as we expect from iterModules on a
        package, whether zipped or not, even if the package has already been
        imported.
        """
        self._setupSysPath()
        namedAny(self.packageName)
        self._listModules()



    def test_moduleAttributes(self):
        """
        Module attributes can be iterated over without executing the code.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".a")
        attrs = sorted(modinfo.iterAttributes(), key=lambda a: a.name)
        names = sorted(["foo", "doFoo", "Foo"])
        for attr, name in zip(attrs, names):
            self.assertEquals(attr.onObject, modinfo)
            self.assertFalse(attr.isLoaded())
            self.assertEquals(attr.name, modinfo.name + '.' + name)
            self.assertRaises(NotImplementedError, lambda: list(attr.iterAttributes()))


    def test_loadedModuleAttributes(self):
        """
        Module attributes can be iterated over after the module has been loaded.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".a")
        modinfo.load()
        attrs = sorted([a for a in modinfo.iterAttributes() if '_' not in a.name], key=lambda a: a.name)
        names = sorted(["foo", "doFoo", "Foo"])
        for attr, name in zip(attrs, names):
            self.assertEquals(attr.onObject, modinfo)
            self.assertTrue(attr.isLoaded())
            self.assertEquals(attr.name, modinfo.name + '.' + name)
            if name == "Foo":
                classattrs = [a.name for a in attr.iterAttributes()]
                self.assertIn(modinfo.name + '.Foo.x', classattrs)


    def test_attributeLoading(self):
        """
        Calling .load() on a L{PythonAttribute} loads the module it's a part of and returns the attribute value.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".a")
        attr = [x for x in modinfo.iterAttributes() if x.name.endswith('foo')][0]

        mod = attr.onObject
        self.assertFalse(attr.isLoaded())
        self.assertFalse(mod.isLoaded())
        val = attr.load()
        self.assertTrue(attr.isLoaded())
        self.assertTrue(mod.isLoaded())
        self.assertEquals(val, 123)
        self.assertEquals(attr.pythonValue, 123)
        self.assertEquals(attr.load(), 123)


    def test_moduleImportNames(self):
        """
        The fully qualified names imported by a module can be inspected.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".a")
        self.assertEquals(sorted(modinfo.iterImportNames()),
                         sorted(["sys", "os", "datetime",
                                 "twisted.python.reflect",
                                 "twisted.python.filepath",
                                 "twisted.python.components.registerAdapter"]))


    def test_moduleExportDefinedNames(self):
        """
        The exports of a module with no __all__ are all its defined
        names.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".a")
        self.assertEqual(sorted(modinfo.iterExportNames()),
                         sorted(["foo", "doFoo", "Foo"]))


    def test_moduleExportAll(self):
        """
        If __all__ is defined as a list of string literals, the names
        in it are used as the list of the module's exports.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".b")
        self.assertEqual(sorted(modinfo.iterExportNames()),
                         sorted(["foo"]))


    def test_exportedImports(self):
        """
        If __all__ mentions imports, they're included in the collection of defined names.
        """
        self._setupSysPath()
        modinfo = modules.getModule(self.packageName + ".d")
        self.assertEqual(sorted([x.name.rsplit('.', 1)[1] for x in
                                 modinfo.iterAttributes()]),
                         sorted(["foo", "registerAdapter", "reflect"]))


    def test_moduleExportProblems(self):
        """
        C{SyntaxError} is raised when doing inspection of module
        exports if __all__ is not a single list of string literals.
        """
        self.packagePath.child("e.py").setContent("__all__ = ['a' + 'b']")
        self.packagePath.child("f.py").setContent("__all__ = ['a']\n__all__ = ['a', 'b']")
        self._setupSysPath()
        modinfo1 = modules.getModule(self.packageName + ".e")
        modinfo2 = modules.getModule(self.packageName + ".f")
        self.assertRaises(SyntaxError, lambda: list(modinfo1.iterExportNames()))
        self.assertRaises(SyntaxError, lambda: list(modinfo2.iterExportNames()))


    if ast is None:
        astMsg = ("Examining unloaded module attributes requires the 'ast'"
                 " module from Python 2.6.")
        test_moduleAttributes.skip = astMsg
        test_moduleImportNames.skip = astMsg
        test_moduleExportAll.skip = astMsg
        test_moduleExportDefinedNames.skip = astMsg
        test_moduleExportProblems.skip = astMsg


    def tearDown(self):
        # Intentionally using 'assert' here, this is not a test assertion, this
        # is just an "oh fuck what is going ON" assertion. -glyph
        if self.pathSetUp:
            HORK = "path cleanup failed: don't be surprised if other tests break"
            assert sys.path.pop() is self.pathExtensionName, HORK+", 1"
            assert self.pathExtensionName not in sys.path, HORK+", 2"



class RebindingTest(PathModificationTest):
    """
    These tests verify that the default path interrogation API works properly
    even when sys.path has been rebound to a different object.
    """
    def _setupSysPath(self):
        assert not self.pathSetUp
        self.pathSetUp = True
        self.savedSysPath = sys.path
        sys.path = sys.path[:]
        sys.path.append(self.pathExtensionName)


    def tearDown(self):
        """
        Clean up sys.path by re-binding our original object.
        """
        if self.pathSetUp:
            sys.path = self.savedSysPath



class ZipPathModificationTest(PathModificationTest):
    def _setupSysPath(self):
        assert not self.pathSetUp
        zipit(self.pathExtensionName, self.pathExtensionName+'.zip')
        self.pathExtensionName += '.zip'
        assert zipfile.is_zipfile(self.pathExtensionName)
        PathModificationTest._setupSysPath(self)


class PythonPathTestCase(TestCase):
    """
    Tests for the class which provides the implementation for all of the
    public API of L{modules}, L{PythonPath}.
    """
    def test_unhandledImporter(self):
        """
        Make sure that the behavior when encountering an unknown importer
        type is not catastrophic failure.
        """
        class SecretImporter(object):
            pass

        def hook(name):
            return SecretImporter()

        syspath = ['example/path']
        sysmodules = {}
        syshooks = [hook]
        syscache = {}
        def sysloader(name):
            return None
        space = modules.PythonPath(
            syspath, sysmodules, syshooks, syscache, sysloader)
        entries = list(space.iterEntries())
        self.assertEquals(len(entries), 1)
        self.assertRaises(KeyError, lambda: entries[0]['module'])


    def test_inconsistentImporterCache(self):
        """
        If the path a module loaded with L{PythonPath.__getitem__} is not
        present in the path importer cache, a warning is emitted, but the
        L{PythonModule} is returned as usual.
        """
        space = modules.PythonPath([], sys.modules, [], {})
        thisModule = space[__name__]
        warnings = self.flushWarnings([self.test_inconsistentImporterCache])
        self.assertEquals(warnings[0]['category'], UserWarning)
        self.assertEquals(
            warnings[0]['message'],
            FilePath(modules.__file__).parent().dirname() +
            " (for module " + __name__ + ") not in path importer cache "
            "(PEP 302 violation - check your local configuration).")
        self.assertEquals(len(warnings), 1)
        self.assertEquals(thisModule.name, __name__)



class ASTVisitorTests(TestCase):
    """
    Tests for L{ast.NodeVisitor} subclasses used to extract
    information from modules without importing them.
    """

    def test_all(self):
        """
        If a module's '__all__' attribute contains a sequence of
        literal strings, they are collected as export names.
        """

        bits = ["foo = 1\n__all__ = ['foo', 'baz']",
                "__all__ = ('foo', 'baz')\nbaz = 1"]

        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            f.visit(tree)
            self.assertEqual(f.exports, set(["foo", "baz"]))


    def test_justOneAll(self):
        """
        Modules with more than one definition of __all__ are rejected
        by L{_ImportExportFinder}.
        """

        bits  = ["__all__ = ['a']\n__all__ = ['a', 'b']",
                 "__all__ = ['a']\n__all__, x = ['a', 'b'], 1",]
        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            self.assertRaises(SyntaxError, f.visit, tree)


    def test_literalStringsAll(self):
        """
        Modules with a definition of __all__ that isn't a list of
        literal strings are rejected by L{_ImportExportFinder}.
        """

        bits = ["__all__ = ['a' + 'b']",
                "__all__ = ['a', 1, 'b']",
                "x, __all__ = ['x', ['a', 1]]"]
        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            self.assertRaises(SyntaxError, f.visit, tree)


    def test_identifierStringsAll(self):
        """
        Modules with a definition of __all__ that contains strings
        that aren't valid Python identifiers are rejected by
        L{_ImportExportFinder}.
        """
        bits = ["__all__ = ['a nice variable', 'etc']",
                "__all__ = ['call-with-current-continuation', 'goto']"]

        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            self.assertRaises(SyntaxError, f.visit, tree)


    def test_multiAssigns(self):
        """
        Assignments to __all__ in a tuple unpacking statement are recognized properly.
        """
        bits = ['__all__, x = (["foo", "baz"], 1)',
                'foo = 1\nx, __all__ = (None, ["foo", "baz"])',
                'x, __all__, y = [[], ["foo", "baz"], {}]']

        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            f.visit(tree)
            self.assertEqual(f.exports, set(["foo", "baz"]))



    def test_bogusMultiAssign(self):
        """
        Tuple-unpacking assigments containing __all__ are rejected if
        the RHS is not a literal sequence of the proper length.
        """
        bits = ["(x, __all__) = 1",
                "[x, __all__] = []",
                "__all__, x = (1,)",
                "(x, (y, __all__)) = 1, 2",
                "(x, (y, __all__)) = (1, (2,))"]

        for code in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            self.assertRaises(SyntaxError, f.visit, tree)

        
    def test_funnyAssigns(self):
        """
        Assignments that aren't to regular names are
        skipped. Assignments to regular names (whether in unpacking
        contexts or not) are recognized.
        """
        bits = [('x[0] = 1', []),
                ('x.y = 2', []),
                ('x, y[0] = 3, 4', ["x"]),
                ('x.y, z = 5, 6', ["z"]),
                ("a, b = 7, 8", ["a", "b"]),
                ("[c, d] = 9, 0", ["c", "d"]),
                ("[e, (f, g)] = 0xA, 0xB", ["e", "f", "g"])]

        for (code, expectedNames) in bits:
            tree = ast.parse(code)
            f = _ImportExportFinder()
            f.visit(tree)
            self.assertEqual(f.definedNames, set(expectedNames), "expected %s as defined names in %r" % (expectedNames, code))


    def test_noImportStar(self):
        """
        Code containing 'import *' is rejected.
        """
        tree = ast.parse("from pickle import *")
        f = _ImportExportFinder()
        self.assertRaises(SyntaxError, f.visit, tree)



if ast is None:
    ASTVisitorTests.skip = "AST visitor tests require the 'ast' module from Python 2.6."


class MiscTests(TestCase):
    def test_isPythonIdentifier(self):
        """
        L{modules._isPythonIdentifier} correctly identifies which strings are
        and aren't valid python identifiers.
        """
        good = ["__init__", "foo", "_foo", "foo_baz", "Foo", "FooBaz_", "foo1", "foo_1", "F00"]
        bad = ["1f", "-foo", "foo-baz", "a foo", "foo.baz", 3, ["foo"], ()]

        for n in good:
            self.assertTrue(_isPythonIdentifier(n))

        for n in bad:
            self.assertFalse(_isPythonIdentifier(n))




# Copyright (c) 2010-2011 Allen Short. See LICENSE file for details.
from exocet._exocet import (load, loadNamed, loadPackage, loadPackageNamed,
                            proxyModule, emptyMapper, pep302Mapper, IMapper,
                            DictMapper, ExclusiveMapper, CallableMapper,
                            getModule)

__all__= ['load', 'loadNamed', 'loadPackage', 'loadPackageNamed',
          'getModule', 'proxyModule', 'emptyMapper', 'pep302Mapper',
          'IMapper', 'DictMapper', 'CallableMapper']

__version__ = '0.5'




from math import cos, sin

def rotated_cosine(x, y, theta, lambd):
    r"""
    Evaluate a rotated 3D sinusoidal wave at a given point, angle, and
    wavelength.

    The function used is:

    .. math::

       f(x, y) = -\cos((x \cos\theta - y \sin\theta) / \lambda) / 2 + 1

    This function has a handful of useful properties; it has a local minimum
    at f(0, 0) and oscillates infinitely betwen 0 and 1.

    :param float x: X coordinate
    :param float y: Y coordinate
    :param float theta: angle of rotation
    :param float lambda: wavelength

    :returns: float of f(x, y)
    """

    return -cos((x * cos(theta) - y * sin(theta)) / lambd) / 2 + 1

def morton2(x, y):
    """
    Create a Morton number by interleaving the bits of two numbers.

    This can be used to map 2D coordinates into the integers.

    Inputs will be masked off to 16 bits, unsigned.
    """

    gx = x & 0xffff
    gy = y & 0xffff

    b = 0x00ff00ff, 0x0f0f0f0f, 0x33333333, 0x55555555
    s = 8, 4, 2, 1

    for i, j in zip(b, s):
        gx = (gx | (gx << j)) & i
        gy = (gy | (gy << j)) & i

    return gx | (gy << 1)



import unittest

from bravo.blocks import blocks, items
from bravo.inventory import Slot
from bravo.policy.dig import dig_policies

class TestNotchyDigPolicy(unittest.TestCase):

    def setUp(self):
        self.p = dig_policies["notchy"]

    def test_trivial(self):
        pass

    def test_sapling_1ko(self):
        self.assertTrue(self.p.is_1ko(blocks["sapling"].slot, None))

    def test_snow_1ko(self):
        """
        Snow can't be 1KO'd by hand, just with a shovel.
        """

        slot = Slot(items["wooden-shovel"].slot, 0x64, 1)

        self.assertFalse(self.p.is_1ko(blocks["snow"].slot, None))
        self.assertTrue(self.p.is_1ko(blocks["snow"].slot, slot))

    def test_dirt_bare(self):
        self.assertAlmostEqual(self.p.dig_time(blocks["dirt"].slot, None),
                               0.75)



import functools

from construct import Struct, Container, Embed, MetaField
from construct import Switch, Const, Peek
from construct import OptionalGreedyRange
from construct import PascalString
from construct import UBInt8, UBInt16, UBInt32

DUMP_ALL_PACKETS = False

AlphaString = functools.partial(PascalString,
    length_field=UBInt16("length"),
    encoding="utf8")

def String(name):
    """
    UTF-8 length-prefixed string.
    """

    return PascalString(name, length_field=UBInt16("length"),
        encoding="utf-8")

def InfiniPacket(name, identifier, subconstruct):
    """
    Common header structure for packets.

    This is possibly not the best way to go about building these kinds of
    things.
    """

    header = Struct("header",
        # XXX Should this be Magic(chr(identifier))?
        Const(UBInt8("identifier"), identifier),
        UBInt8("flags"),
        UBInt32("length"),
    )

    return Struct(name, header, subconstruct)

packets = {
    0: InfiniPacket("ping", 0x00,
        Struct("payload",
            UBInt16("uid"),
            UBInt32("timestamp"),
        )
    ),
    255: InfiniPacket("disconnect", 0xff,
        Struct("payload",
            AlphaString("explanation"),
        )
    ),
    "__default__": Struct("unknown",
        Struct("header",
            UBInt8("identifier"),
            UBInt8("flags"),
            UBInt32("length"),
        ),
        MetaField("data", lambda context: context["length"]),
    ),
}

packets_by_name = {
    "ping"       : 0,
    "disconnect" : 255,
}

infinipacket_parser = Struct("parser",
    OptionalGreedyRange(
        Struct("packets",
            Peek(UBInt8("header")),
            Embed(Switch("packet", lambda context: context["header"],
                packets)),
        ),
    ),
    OptionalGreedyRange(
        UBInt8("leftovers"),
    ),
)

def parse_packets(bytestream):
    container = infinipacket_parser.parse(bytestream)

    l = [(i.header, i.payload) for i in container.packets]
    leftovers = "".join(chr(i) for i in container.leftovers)

    if DUMP_ALL_PACKETS:
        for packet in l:
            print "Parsed packet %d" % packet[0]
            print packet[1]

    return l, leftovers

def make_packet(packet, *args, **kwargs):
    """
    Constructs a packet bytestream from a packet header and payload.

    The payload should be passed as keyword arguments. Additional containers
    or dictionaries to be added to the payload may be passed positionally, as
    well.
    """

    if packet not in packets_by_name:
        print "Couldn't find packet name %s!" % packet
        return ""

    header = packets_by_name[packet]

    for arg in args:
        kwargs.update(dict(arg))
    payload = Container(**kwargs)

    if DUMP_ALL_PACKETS:
        print "Making packet %s (%d)" % (packet, header)
        print payload
    payload = packets[header].build(payload)
    return chr(header) + payload



from collections import defaultdict
from itertools import product
from UserDict import DictMixin

from bravo.utilities.coords import taxicab2

class SpatialDict(object, DictMixin):
    """
    A spatial dictionary, for accelerating spatial lookups.

    This particular class is a template for specific spatial dictionaries; in
    order to make it work, subclass it and add ``key_for_bucket()``.
    """

    def __init__(self):
        self.buckets = defaultdict(dict)

    def __setitem__(self, key, value):
        """
        Add a key-value pair to the dictionary.

        :param tuple key: a tuple of (x, z) coordinates
        :param object value: an object
        """

        bucket_key = self.key_for_bucket(key)
        self.buckets[bucket_key][key] = value

    def __getitem__(self, key):
        """
        Retrieve a value, given a key.
        """

        bucket_key = self.key_for_bucket(key)
        return self.buckets[bucket_key][key]

    def __delitem__(self, key):
        """
        Remove a key and its corresponding value.
        """

        bucket_key = self.key_for_bucket(key)
        del self.buckets[bucket_key][key]

        if not self.buckets[bucket_key]:
            del self.buckets[bucket_key]

    def iterkeys(self):
        """
        Yield all the keys.
        """

        for bucket in self.buckets.itervalues():
            for key in bucket.iterkeys():
                yield key

    def keys(self):
        """
        Get a list of all keys in the dictionary.
        """

        return list(self.iterkeys())

    def iteritemsnear(self, key, radius):
        """
        A version of ``iteritems()`` that filters based on the distance from a
        given key.

        The key does not need to actually be in the dictionary.
        """

        for coords in self.keys_near(key, radius):
            for target, value in self.buckets[coords].iteritems():
                if taxicab2(target[0], target[1], key[0], key[1]) <= radius:
                    yield target, value

    def iterkeysnear(self, key, radius):
        """
        Yield all of the keys within a certain radius of this key.
        """

        for k, v in self.iteritemsnear(key, radius):
            yield k

    def itervaluesnear(self, key, radius):
        """
        Yield all of the values within a certain radius of this key.
        """

        for k, v in self.iteritemsnear(key, radius):
            yield v

class Block2DSpatialDict(SpatialDict):
    """
    Class for tracking blocks in the XZ-plane.
    """

    def key_for_bucket(self, key):
        """
        Partition keys into chunk-sized buckets.
        """

        try:
            return int(key[0] // 16), int(key[1] // 16)
        except ValueError:
            return KeyError("Key %s isn't usable here!" % repr(key))

    def keys_near(self, key, radius):
        """
        Get all bucket keys "near" this key.

        This method may return a generator.
        """

        minx, innerx = divmod(key[0], 16)
        minz, innerz = divmod(key[1], 16)
        minx = int(minx)
        minz = int(minz)

        # Adjust for range() purposes.
        maxx = minx + 1
        maxz = minz + 1

        # Adjust for leakiness.
        if innerx <= radius:
            minx -= 1
        if innerz <= radius:
            minz -= 1
        if innerx + radius >= 16:
            maxx += 1
        if innerz + radius >= 16:
            maxz += 1

        # Expand as needed.
        expand = int(radius // 16)
        minx -= expand
        minz -= expand
        maxx += expand
        maxz += expand

        return product(xrange(minx, maxx), xrange(minz, maxz))

class Block3DSpatialDict(SpatialDict):
    """
    Class for tracking blocks in the XZ-plane.
    """

    def key_for_bucket(self, key):
        """
        Partition keys into chunk-sized buckets.
        """

        try:
            return int(key[0] // 16), int(key[1] // 16), int(key[2] // 16)
        except ValueError:
            return KeyError("Key %s isn't usable here!" % repr(key))

    def keys_near(self, key, radius):
        """
        Get all bucket keys "near" this key.

        This method may return a generator.
        """

        minx, innerx = divmod(key[0], 16)
        miny, innery = divmod(key[1], 16)
        minz, innerz = divmod(key[2], 16)
        minx = int(minx)
        miny = int(miny)
        minz = int(minz)

        # Adjust for range() purposes.
        maxx = minx + 1
        maxy = miny + 1
        maxz = minz + 1

        # Adjust for leakiness.
        if innerx <= radius:
            minx -= 1
        if innery <= radius:
            miny -= 1
        if innerz <= radius:
            minz -= 1
        if innerx + radius >= 16:
            maxx += 1
        if innery + radius >= 16:
            maxy += 1
        if innerz + radius >= 16:
            maxz += 1

        # Expand as needed.
        expand = int(radius // 16)
        minx -= expand
        miny -= expand
        minz -= expand
        maxx += expand
        maxy += expand
        maxz += expand

        return product(
            xrange(minx, maxx),
            xrange(miny, maxy),
            xrange(minz, maxz))



from zope.interface import implements

from bravo.ibravo import IConsoleCommand, IChatCommand

from bravo.parameters import factory

class Meliae(object):

    implements(IConsoleCommand)

    def console_command(self, parameters):
        out = "".join(parameters)
        try:
            import meliae.scanner
            meliae.scanner.dump_all_objects(out)
        except ImportError:
            raise Exception("Couldn't import meliae!")
        except IOError:
            raise Exception("Couldn't save to file %s!" % parameters)

        return tuple()

    name = "dump-memory"
    aliases = tuple()
    usage = "<filename>"
    info = "Dump a JSON snapshot of memory usage using Meliae"

class Status(object):

    implements(IConsoleCommand)

    def console_command(self, parameters):
        protocol_count = len(factory.protocols)
        yield "%d protocols connected" % protocol_count

        for name, protocol in factory.protocols.iteritems():
            count = len(protocol.chunks)
            dirty = len([i for i in protocol.chunks.values() if i.dirty])
            yield "%s: %d chunks (%d dirty)" % (name, count, dirty)

        chunk_count = len(factory.world.chunk_cache)
        dirty = len(factory.world.dirty_chunk_cache)
        chunk_count += dirty
        yield "World cache: %d chunks (%d dirty)" % (chunk_count, dirty)

    name = "status"
    aliases = tuple()
    usage = ""
    info = "Print a quick summary of the server's status"

class Colors(object):

    implements(IChatCommand)

    def chat_command(self, username, parameters):
        from bravo.utilities.chat import chat_colors
        names = """black dblue dgreen dcyan dred dmagenta dorange gray dgray
        blue green cyan red magenta yellow""".split()
        for pair in zip(chat_colors, names):
            yield "%s%s" % pair

    name = "colors"
    aliases = tuple()
    usage = ""
    info = "Print the colors"

meliae = Meliae()
status = Status()
colors = Colors()



from gzip import GzipFile
from StringIO import StringIO
import tempfile
import unittest

from bravo.nbt import NBTFile, MalformedFileError
from bravo.nbt import TAG_Compound

bigtest = """
H4sIAAAAAAAAAO1Uz08aQRR+wgLLloKxxBBjzKu1hKXbzUIRibGIFiyaDRrYqDGGuCvDgi67Znew
8dRLe2x66z/TI39Dz732v6DDL3tpz73wMsn35r1v5ntvJnkCBFRyTywOeMuxTY149ONwYj4Iex3H
pZMYD4JH3e6EAmK1oqrHeHZcV8uoVQ8byNYeapWGhg2tflh7j4PPg0+Db88DEG5bjj6+pThMZP0Q
6tp0piNA3GYuaeG107tz+nYLKdsL4O/oPR44W+8RCFb13l3fC0DgXrf6ZLcEAIxBTHPGCFVM0yAu
faTAyMIQs7reWAtTo+5EjkUDMLEnU4xM8ekUo1OMheHZn+Oz8kSBpXwz3di7x6p1E18oHAjXLtFZ
P68dG2AhWd/68QX+wc78nb0AvPFAyfiFQkBG/p7r6g+TOmiHYLvrMjejKAqOu/XQaWPKTtvp7Obm
Kzu9Jb5kSQk9qruU/Rh+6NIO2m8VTLFoPivhm5yEmbyEBQllWRZFAP8vKK4v8sKypC4dIHdaO7mM
yucp31FByRa1xW2hKq0sxTF/unqSjl6dX/gSBSMb0fa3d6rNlXK8nt9YXUuXrpIXuUTQgMj6Pr+z
3FTLB3Vuo7Z2WZKTqdxRUJlrzDXmGv9XIwhCy+kb1njC7P78evt9eNOE39TypPsIBgAA
""".decode("base64")

class BugfixTest(unittest.TestCase):
    """
    Bugfix regression tests.

    These tend to not fit into nice categories.
    """

    def test_empty_file(self):
        """
        Opening an empty file causes an exception.

        https://github.com/twoolie/NBT/issues/issue/4
        """

        temp = tempfile.NamedTemporaryFile()
        temp.flush()
        self.assertRaises(MalformedFileError, NBTFile, temp.name)

class ReadWriteTest(unittest.TestCase):

    def setUp(self):
        self.f = tempfile.NamedTemporaryFile()
        self.f.write(bigtest)
        self.f.flush()

    def test_trivial(self):
        pass

    def testReadBig(self):
        mynbt = NBTFile(self.f.name)
        self.assertTrue(mynbt.filename != None)
        self.assertEqual(len(mynbt.tags), 11)

    def testWriteBig(self):
        mynbt = NBTFile(self.f.name)
        output = StringIO()
        mynbt.write_file(buffer=output)
        self.assertTrue(GzipFile(self.f.name).read() == output.getvalue())

    def testWriteback(self):
        mynbt = NBTFile(self.f.name)
        mynbt.write_file()

class TreeManipulationTest(unittest.TestCase):

    def setUp(self):
        self.nbtfile = NBTFile()

    def testRootNodeSetup(self):
        self.nbtfile.name = "Hello World"
        self.assertEqual(self.nbtfile.name, "Hello World")

class EmptyStringTest(unittest.TestCase):

    def setUp(self):
        self.golden_value = "\x0A\0\x04Test\x08\0\x0Cempty string\0\0\0"
        self.nbtfile = NBTFile(buffer=StringIO(self.golden_value))

    def testReadEmptyString(self):
        self.assertEqual(self.nbtfile.name, "Test")
        self.assertEqual(self.nbtfile["empty string"].value, "")

    def testWriteEmptyString(self):
        buffer = StringIO()
        self.nbtfile.write_file(buffer=buffer)
        self.assertEqual(buffer.getvalue(), self.golden_value)

class TestTAGCompound(unittest.TestCase):

    def setUp(self):
        self.tag = TAG_Compound()

    def test_trivial(self):
        pass

    def test_contains(self):
        self.tag["test"] = TAG_Compound()
        self.assertTrue("test" in self.tag)

if __name__ == '__main__':
    unittest.main()



from math import cos, sin

from zope.interface import implements

from bravo.blocks import blocks
from bravo.ibravo import IChatCommand, IConsoleCommand
from bravo.utilities.geometry import gen_line_covered

from bravo.parameters import factory

"""
This plugin shall mimic the worldedit plugin for the original minecraft_server.jar.
"""

empty_blocks_names = (
    "air", "snow", "sapling", "water", "spring",
    "flower", "rose", "brown-mushroom", "red-mushroom",
    "torch", "fire", "redstone-wire", "crops", "soil", "signpost",
    "wooden-door", "iron-door", "wall-sign", "lever",
    "stone-plate", "wooden-plate", "sugar-cane", "fence", "portal",
    "redstone-repeater-on", "redstone-repeater-off",
)

empty_blocks = []
for name in empty_blocks_names:
    empty_blocks.append(blocks[name].slot)

class _Point(object):
    """
    Small temporary class to hold a 3d point.
    """

    def __init__ (self, vec):
        self.x, self.y, self.z = vec

class Jumpto(object):
    """
    Teleport the player to the block he's looking at.
    """

    implements(IChatCommand, IConsoleCommand)

    def chat_command(self, username, parameters):
        yield "Trying to Jump..."

        protocol = factory.protocols[username] # Our player object

        l = protocol.player.location
        # Viewport and player location are not very adapted to the coordinate system :
        # Blocks are not aligned on their centers. To be standing in the middle of a block,
        # you have to be at +0.5 +0.5 on x and z positions.
        o = _Point((l.x, l.y + 1.6, l.z))

        # x = r sinq cosf,     y = r sinq sinf,     z = r cosq,
        distant_point = _Point((-1 * 220 * cos(l.phi) * sin(l.theta),
            -1 * 220 * sin(l.phi), 220 * cos(l.theta) * cos(l.phi)))
        distant_point.x += o.x
        distant_point.y += o.y
        distant_point.z += o.z

        world = factory.world
        dest = None
        for point in gen_line_covered(o, distant_point):
            block = world.get_block(point)
            if block.result not in empty_blocks: # it's not air !!
                dest = [point[0], point[1], point[2]]
                break

        if not dest:
            yield "Could not find a suitable destination..."
            return

        # Now we find the first vertical space that can host us.
        current_block = -1
        prev_block = -1
        while True: # Should include non block as well !
            prev_block = current_block
            dest[1] += 1
            if dest[1] >= 127:
                dest[1] += 1
                break
            current_block = world.get_block(dest).result
            if current_block in empty_blocks and prev_block in empty_blocks:
                break

        l.x, l.y, l.z = dest[0] + 0.5, dest[1], dest[2] + 0.5
        protocol.send_initial_chunk_and_location()
        yield "*Poof*"

    name = "jumpto"
    aliases = tuple()
    usage = "<name>"
    info = "Teleports you where you're looking at."

jumpto = Jumpto()



from twisted.trial import unittest

from twisted.internet.defer import inlineCallbacks

import numpy
import shutil
import tempfile

from itertools import product

import bravo.config
from bravo.errors import ChunkNotLoaded, SerializerReadException
from bravo.world import World

class TestWorldChunks(unittest.TestCase):

    def setUp(self):
        self.name = "unittest"
        self.d = tempfile.mkdtemp()

        bravo.config.configuration.add_section("world unittest")
        bravo.config.configuration.set("world unittest", "url", "file://%s" % self.d)
        bravo.config.configuration.set("world unittest", "serializer",
            "alpha")

        self.w = World(self.name)
        self.w.pipeline = []
        self.w.start()

    def tearDown(self):
        self.w.stop()
        del self.w

        shutil.rmtree(self.d)
        bravo.config.configuration.remove_section("world unittest")

    def test_trivial(self):
        pass

    @inlineCallbacks
    def test_get_block(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Fill the chunk with random stuff.
        chunk.blocks = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.blocks.shape = (16, 16, 128)

        for x, y, z in product(xrange(2), repeat=3):
            # This works because the chunk is at (0, 0) so the coords don't
            # need to be adjusted.
            block = yield self.w.get_block((x, y, z))
            self.assertEqual(block, chunk.get_block((x, y, z)))

    @inlineCallbacks
    def test_get_metadata(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Fill the chunk with random stuff.
        chunk.metadata = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.metadata.shape = (16, 16, 128)

        for x, y, z in product(xrange(2), xrange(2), xrange(2)):
            # This works because the chunk is at (0, 0) so the coords don't
            # need to be adjusted.
            metadata = yield self.w.get_metadata((x, y, z))
            self.assertEqual(metadata, chunk.get_metadata((x, y, z)))

    @inlineCallbacks
    def test_get_block_readback(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Fill the chunk with random stuff.
        chunk.blocks = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.blocks.shape = (16, 16, 128)

        # Evict the chunk and grab it again.
        self.w.save_chunk(chunk)
        del chunk
        self.w.chunk_cache.clear()
        self.w.dirty_chunk_cache.clear()
        chunk = yield self.w.request_chunk(0, 0)

        for x, y, z in product(xrange(2), xrange(2), xrange(2)):
            # This works because the chunk is at (0, 0) so the coords don't
            # need to be adjusted.
            block = yield self.w.get_block((x, y, z))
            self.assertEqual(block, chunk.get_block((x, y, z)))

    @inlineCallbacks
    def test_get_block_readback_negative(self):
        chunk = yield self.w.request_chunk(-1, -1)

        # Fill the chunk with random stuff.
        chunk.blocks = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.blocks.shape = (16, 16, 128)

        # Evict the chunk and grab it again.
        self.w.save_chunk(chunk)
        del chunk
        self.w.chunk_cache.clear()
        self.w.dirty_chunk_cache.clear()
        chunk = yield self.w.request_chunk(-1, -1)

        for x, y, z in product(xrange(2), xrange(2), xrange(2)):
            block = yield self.w.get_block((x - 16, y, z - 16))
            self.assertEqual(block, chunk.get_block((x, y, z)))

    @inlineCallbacks
    def test_get_metadata_readback(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Fill the chunk with random stuff.
        chunk.metadata = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.metadata.shape = (16, 16, 128)

        # Evict the chunk and grab it again.
        self.w.save_chunk(chunk)
        del chunk
        self.w.chunk_cache.clear()
        self.w.dirty_chunk_cache.clear()
        chunk = yield self.w.request_chunk(0, 0)

        for x, y, z in product(xrange(2), xrange(2), xrange(2)):
            # This works because the chunk is at (0, 0) so the coords don't
            # need to be adjusted.
            metadata = yield self.w.get_metadata((x, y, z))
            self.assertEqual(metadata, chunk.get_metadata((x, y, z)))

    @inlineCallbacks
    def test_world_level_mark_chunk_dirty(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Reload chunk.
        self.w.save_chunk(chunk)
        del chunk
        self.w.chunk_cache.clear()
        self.w.dirty_chunk_cache.clear()
        chunk = yield self.w.request_chunk(0, 0)

        self.assertFalse(chunk.dirty)
        self.w.mark_dirty((12, 64, 4))
        chunk = yield self.w.request_chunk(0, 0)
        self.assertTrue(chunk.dirty)

    @inlineCallbacks
    def test_world_level_mark_chunk_dirty_offset(self):
        chunk = yield self.w.request_chunk(1, 2)

        # Reload chunk.
        self.w.save_chunk(chunk)
        del chunk
        self.w.chunk_cache.clear()
        self.w.dirty_chunk_cache.clear()
        chunk = yield self.w.request_chunk(1, 2)

        self.assertFalse(chunk.dirty)
        self.w.mark_dirty((29, 64, 43))
        chunk = yield self.w.request_chunk(1, 2)
        self.assertTrue(chunk.dirty)

    @inlineCallbacks
    def test_sync_get_block(self):
        chunk = yield self.w.request_chunk(0, 0)

        # Fill the chunk with random stuff.
        chunk.blocks = numpy.fromstring(numpy.random.bytes(chunk.blocks.size),
            dtype=numpy.uint8)
        chunk.blocks.shape = (16, 16, 128)

        for x, y, z in product(xrange(2), repeat=3):
            # This works because the chunk is at (0, 0) so the coords don't
            # need to be adjusted.
            block = self.w.sync_get_block((x, y, z))
            self.assertEqual(block, chunk.get_block((x, y, z)))

    def test_sync_get_block_unloaded(self):
        self.assertRaises(ChunkNotLoaded, self.w.sync_get_block, (0, 0, 0))

    def test_sync_get_metadata_neighboring(self):
        """
        Even if a neighboring chunk is loaded, the target chunk could still be
        unloaded.

        Test with sync_get_metadata() to increase test coverage.
        """

        d = self.w.request_chunk(0, 0)

        @d.addCallback
        def cb(chunk):
            self.assertRaises(ChunkNotLoaded,
                              self.w.sync_get_metadata, (16, 0, 0))

        return d

class TestWorldInit(unittest.TestCase):

    def setUp(self):
        self.name = "unittest"
        self.d = tempfile.mkdtemp()

        bravo.config.configuration.add_section("world unittest")
        bravo.config.configuration.set("world unittest", "url", "file://%s" % self.d)
        bravo.config.configuration.set("world unittest", "serializer",
            "alpha")

    def tearDown(self):
        shutil.rmtree(self.d)
        bravo.config.configuration.remove_section("world unittest")

    def test_trivial(self):
        pass

    def test_load_level_exception(self):
        """
        Exceptions raised during serializer reads should be handled during the
        world startup.
        """

        from bravo.plugins.serializers import Alpha
        def raiser(self, level):
            raise SerializerReadException("testing")
        self.patch(Alpha, "load_level", raiser)

        w = bravo.world.World(self.name)
        w.start()
        w.stop()



from functools import wraps
from time import time

"""
Decorators.
"""

timers = {}

def timed(f):
    timers[f] = (0, 0)
    @wraps(f)
    def deco(*args, **kwargs):
        before = time()
        retval = f(*args, **kwargs)
        after = time()
        count, average = timers[f]
        # MMA
        average = (9 * average + after - before) / 10
        count += 1
        if not count % 10:
            print "Average time for %s: %dms" % (f, average * 1000)
        timers[f] = (count, average)
        return retval
    return deco



from itertools import product

from numpy import transpose

def naive_scan(automaton, chunk):
    """
    Utility function which can be used to implement a naive, slow, but
    thorough chunk scan for automatons.

    This method is designed to be directly useable on automaton classes to
    provide the `scan()` interface.
    """

    for block in automaton.blocks:
        for coords in transpose((chunk.blocks == block).nonzero()):
            # Swizzle and discard numpy-ness.
            coords = coords[0], coords[2], coords[1]
            automaton.feed(coords)

def column_scan(automaton, chunk):
    """
    Utility function which provides a chunk scanner which only examines the
    tallest blocks in the chunk. This can be useful for automatons which only
    care about sunlit or elevated areas.

    This method can be used directly in automaton classes to provide `scan()`.
    """

    for x, z in product(range(16), repeat=2):
        y = chunk.height_at(x, z)
        if chunk.get_block((x, y, z)) in automaton.blocks:
            automaton.feed((x + chunk.x * 16, y, z + chunk.z * 16))



from types import ModuleType
from xml.sax import saxutils

from exocet import ExclusiveMapper, getModule, load, pep302Mapper

from twisted.internet import reactor
from twisted.plugin import IPlugin
from twisted.python import log

from zope.interface.exceptions import BrokenImplementation
from zope.interface.exceptions import BrokenMethodImplementation
from zope.interface.verify import verifyObject

from bravo.errors import PluginException
from bravo.ibravo import InvariantException, ISortedPlugin

blacklisted = set([
    "asyncore",        # Use Twisted's event loop.
    "ctypes",          # Segfault protection.
    "gc",              # Haha, no.
    "imp",             # Haha, no.
    "inspect",         # Haha, no.
    "multiprocessing", # Use Twisted's process interface.
    "socket",          # Use Twisted's socket interface.
    "subprocess",      # Use Twisted's process interface.
    "thread",          # Use Twisted's thread interface.
    "threading",       # Use Twisted's thread interface.
])
overrides = {
    "twisted.internet.reactor": reactor,
    "saxutils": saxutils,
}
bravoMapper = ExclusiveMapper(pep302Mapper,
                              blacklisted).withOverrides(overrides)

def sort_plugins(plugins):
    """
    Make a sorted list of plugins by dependency.

    If the list cannot be arranged into a DAG, an error will be raised. This
    usually means that a cyclic dependency was found.

    :raises PluginException: cyclic dependency detected
    """

    l = []
    d = dict((plugin.name, plugin) for plugin in plugins)

    def visit(plugin):
        if plugin not in l:
            for name in plugin.before:
                if name in d:
                    visit(d[name])
            l.append(plugin)

    for plugin in plugins:
        if not any(name in d for name in plugin.after):
            visit(plugin)

    return l

def add_plugin_edges(d):
    """
    Mirror edges to all plugins in a dictionary.
    """

    for plugin in d.itervalues():
        plugin.after = set(plugin.after)
        plugin.before = set(plugin.before)

    for name, plugin in d.iteritems():
        for edge in list(plugin.before):
            if edge in d:
                d[edge].after.add(name)
            else:
                plugin.before.discard(edge)
        for edge in list(plugin.after):
            if edge in d:
                d[edge].before.add(name)
            else:
                plugin.after.discard(edge)

    return d

def expand_names(plugins, names):
    """
    Given a list of names, expand wildcards and discard disabled names.

    Used to implement * and - options in plugin lists.

    :param dict plugins: plugins to use for expansion
    :param list names: names to examine

    :returns: a list of filtered plugin names
    """

    # Wildcard.
    if "*" in names:
        # Get the exceptions.
        exceptions = set(name[1:] for name in names if name.startswith("-"))

        # And now the names. Everything that isn't excepted.
        names = [name for name in plugins.iterkeys()
            if name not in exceptions]

    return names

def verify_plugin(interface, plugin):
    """
    Plugin interface verification.

    This function will call ``verifyObject()`` and ``validateInvariants()`` on
    the plugins passed to it.

    The primary purpose of this wrapper is to do logging, but it also permits
    code to be slightly cleaner, easier to test, and callable from other
    modules.
    """

    try:
        verifyObject(interface, plugin)
        interface.validateInvariants(plugin)
        log.msg(" ( ^^) Plugin: %s" % plugin.name)
    except BrokenImplementation, bi:
        if hasattr(plugin, "name"):
            log.msg(" ( ~~) Plugin %s is missing attribute %r!" %
                (plugin.name, bi.name))
        else:
            log.msg(" ( >&) Plugin %s is unnamed and useless!" % plugin)
    except BrokenMethodImplementation, bmi:
        log.msg(" ( Oo) Plugin %s has a broken %s()!" % (plugin.name,
            bmi.method))
        log.msg(bmi)
    except InvariantException, ie:
        log.msg(" ( >&) Plugin %s failed validation!" % plugin.name)
        log.msg(ie)
    else:
        return plugin

    raise PluginException("Plugin failed verification")

def synthesize_parameters(parameters):
    """
    Create a faked module which has the given parameters in it.

    This should work everywhere. If it doesn't, let me know.
    """

    module = ModuleType("parameters")
    module.__dict__.update(parameters)
    return module

def get_plugins(interface, package, parameters=None):
    """
    Lazily find objects in a package which implement a given interface.

    The objects must also implement ``twisted.plugin.IPlugin``.

    If the optional dictionary of parameters is provided, it will be passed
    into each plugin module as the "bravo.parameters" module. An example
    access from inside the plugin:

    >>> from bravo.parameters import foo, bar

    Since the parameters are available as a real module, the parameters may be
    imported and used like any other module:

    >>> from bravo import parameters as params

    This is a rewrite of Twisted's ``twisted.plugin.getPlugins`` which uses
    Exocet instead of Twisted to find the plugins.

    :param interface interface: the interface to match against
    :param str package: the name of the package to search
    :param dict parameters: parameters to pass into the plugins
    """

    mapper = bravoMapper

    if parameters:
        mapper = mapper.withOverrides(
            {"bravo.parameters": synthesize_parameters(parameters)})

    p = getModule(package)
    for pm in p.iterModules():
        try:
            m = load(pm, mapper)
            for obj in vars(m).itervalues():
                try:
                    adapted = IPlugin(obj, None)
                    adapted = interface(adapted, None)
                except:
                    log.err()
                else:
                    if adapted is not None:
                        yield adapted
        except ImportError, ie:
            log.msg(ie)

def retrieve_plugins(interface, parameters=None):
    """
    Look up all plugins for a certain interface.

    If the plugin cache is enabled, this function will not attempt to reload
    plugins from disk or discover new plugins.

    :param interface interface: the interface to use
    :param dict parameters: parameters to pass into the plugins

    :returns: a dict of plugins, keyed by name
    :raises PluginException: no plugins could be found for the given interface
    """

    log.msg("Discovering %s..." % interface)
    d = {}
    for p in get_plugins(interface, "bravo.plugins", parameters):
        try:
            verify_plugin(interface, p)
            d[p.name] = p
        except PluginException:
            pass

    if issubclass(interface, ISortedPlugin):
        # Sortable plugins need their edges mirrored.
        d = add_plugin_edges(d)

    return d

def retrieve_named_plugins(interface, names, parameters=None):
    """
    Look up a list of plugins by name.

    Plugins are returned in the same order as their names.

    :param interface interface: the interface to use
    :param list names: plugins to find
    :param dict parameters: parameters to pass into the plugins

    :returns: a list of plugins
    :raises PluginException: no plugins could be found for the given interface
    """

    d = retrieve_plugins(interface, parameters)

    # Handle wildcards and options.
    names = expand_names(d, names)

    try:
        return [d[name] for name in names]
    except KeyError, e:
        raise PluginException("Couldn't find plugin %s for interface %s!" %
            (e.args[0], interface.__name__))

def retrieve_sorted_plugins(interface, names, parameters=None):
    """
    Look up a list of plugins, sorted by interdependencies.

    :param dict parameters: parameters to pass into the plugins
    """

    l = retrieve_named_plugins(interface, names, parameters)
    try:
        return sort_plugins(l)
    except KeyError, e:
        raise PluginException("Couldn't find plugin %s for interface %s!" %
            (e.args[0], interface))



from itertools import chain, product

from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import LoopingCall
from zope.interface import implements

from bravo.blocks import blocks
from bravo.ibravo import IAutomaton, IDigHook
from bravo.utilities.automatic import naive_scan
from bravo.utilities.spatial import Block2DSpatialDict, Block3DSpatialDict
from bravo.world import ChunkNotLoaded

from bravo.parameters import factory

FALLING = 0x8
"""
Flag indicating whether fluid is in freefall.
"""

class Fluid(object):
    """
    Fluid simulator.
    """

    implements(IAutomaton, IDigHook)

    sponge = None
    """
    Block that will soak up fluids and springs that are near it.

    Defaults to None, which effectively disables this feature.
    """

    def __init__(self):
        self.sponges = Block3DSpatialDict()
        self.springs = Block2DSpatialDict()

        self.tracked = set()
        self.new = set()

        self.loop = LoopingCall(self.process)

    def start(self):
        if not self.loop.running:
            self.loop.start(self.step)

    def stop(self):
        if self.loop.running:
            self.loop.stop()

    def schedule(self):
        if self.tracked:
            self.start()
        else:
            self.stop()

    @property
    def blocks(self):
        retval = [self.spring, self.fluid]
        if self.sponge:
            retval.append(self.sponge)
        return retval

    def feed(self, coordinates):
        """
        Accept the coordinates and stash them for later processing.
        """

        self.tracked.add(coordinates)
        self.schedule()

    scan = naive_scan

    def update_fluid(self, w, coords, falling, level=0):

        if not 0 <= coords[1] < 128:
            return False

        block = w.sync_get_block(coords)

        if (block in self.whitespace and not
            any(self.sponges.iteritemsnear(coords, 2))):
            w.sync_set_block(coords, self.fluid)
            if falling:
                level |= FALLING
            w.sync_set_metadata(coords, level)
            self.new.add(coords)
            return True
        return False

    def add_sponge(self, w, x, y, z):
        # Track this sponge.
        self.sponges[x, y, z] = True

        # Destroy the water! Destroy!
        for coords in product(
            xrange(x - 2, x + 3),
            xrange(max(y - 2, 0), min(y + 3, 128)),
            xrange(z - 2, z + 3),
            ):
            try:
                target = w.sync_get_block(coords)
                if target == self.spring:
                    if (coords[0], coords[2]) in self.springs:
                        del self.springs[coords[0],
                            coords[2]]
                    w.sync_destroy(coords)
                elif target == self.fluid:
                    w.sync_destroy(coords)
            except ChunkNotLoaded:
                pass

        # And now mark our surroundings so that they can be
        # updated appropriately.
        for coords in product(
            xrange(x - 3, x + 4),
            xrange(max(y - 3, 0), min(y + 4, 128)),
            xrange(z - 3, z + 4),
            ):
            if coords != (x, y, z):
                self.new.add(coords)

    def add_spring(self, w, x, y, z):
        # Double-check that we weren't placed inside a sponge. That's just
        # not going to work out.
        if any(self.sponges.iteritemsnear((x, y, z), 2)):
            w.sync_destroy((x, y, z))
            return

        # Track this spring.
        self.springs[x, z] = y

        # Neighbors on the xz-level.
        neighbors = ((x - 1, y, z), (x + 1, y, z), (x, y, z - 1),
            (x, y, z + 1))
        # Our downstairs pal.
        below = (x, y - 1, z)

        # Spawn water from springs.
        for coords in neighbors:
            try:
                self.update_fluid(w, coords, False)
            except ChunkNotLoaded:
                pass

        # Is this water falling down to the next y-level? We don't really
        # care, but we'll run the update nonetheless.
        self.update_fluid(w, below, True)

    def add_fluid(self, w, x, y, z):
        # Neighbors on the xz-level.
        neighbors = ((x - 1, y, z), (x + 1, y, z), (x, y, z - 1),
                (x, y, z + 1))
        # Our downstairs pal.
        below = (x, y - 1, z)

        # Double-check that we weren't placed inside a sponge.
        if any(self.sponges.iteritemsnear((x, y, z), 2)):
            w.sync_destroy((x, y, z))
            return

        # First, figure out whether or not we should be spreading.  Let's see
        # if there are any springs nearby which are above us and thus able to
        # fuel us.
        if not any(springy >= y
            for springy in
            self.springs.itervaluesnear((x, z), self.levels + 1)):
            # Oh noes, we're drying up! We should mark our neighbors and dry
            # ourselves up.
            self.new.update(neighbors)
            self.new.add(below)
            w.sync_destroy((x, y, z))
            return

        newmd = self.levels + 1

        for coords in neighbors:
            try:
                jones = w.sync_get_block(coords)
                if jones == self.spring:
                    newmd = 0
                    self.new.update(neighbors)
                    break
                elif jones == self.fluid:
                    jonesmd = w.sync_get_metadata(coords) & ~FALLING
                    if jonesmd + 1 < newmd:
                        newmd = jonesmd + 1
            except ChunkNotLoaded:
                pass

        current_md = w.sync_get_metadata((x,y,z))
        if newmd > self.levels and current_md < FALLING:
            # We should dry up.
            self.new.update(neighbors)
            self.new.add(below)
            w.sync_destroy((x, y, z))
            return

        # Mark any neighbors which should adjust themselves. This will only
        # mark lower water levels than ourselves, and only if they are
        # definitely too low.
        for coords in neighbors:
            try:
                neighbor = w.sync_get_metadata(coords)
                if neighbor & ~FALLING > newmd + 1:
                    self.new.add(coords)
            except ChunkNotLoaded:
                pass

        # Now, it's time to extend water. Remember, either the water flows
        # downward to the next y-level, or it flows out across the xz-level,
        # but *not* both.

        # Fall down to the next y-level, if possible.
        if self.update_fluid(w, below, True, newmd):
            return

        # Clamp our newmd and assign. Also, set ourselves again; we changed
        # this time and we might change again.
        if current_md < FALLING:
            w.sync_set_metadata((x, y, z), newmd)

        # If pending block is already above fluid, don't keep spreading.
        if neighbor == self.fluid:
            return

        # Otherwise, just fill our neighbors with water, where applicable, and
        # mark them.
        if newmd < self.levels:
            newmd += 1
            for coords in neighbors:
                try:
                    self.update_fluid(w, coords, False, newmd)
                except ChunkNotLoaded:
                    pass

    def remove_sponge(self, x, y, z):
        # The evil sponge tyrant is gone. Flow, minions, flow!
        for coords in product(xrange(x - 3, x + 4),
            xrange(max(y - 3, 0), min(y + 4, 128)), xrange(z - 3, z + 4)):
            if coords != (x, y, z):
                self.new.add(coords)

    def remove_spring(self, x, y, z):
        # Neighbors on the xz-level.
        neighbors = ((x - 1, y, z), (x + 1, y, z), (x, y, z - 1),
                (x, y, z + 1))
        # Our downstairs pal.
        below = (x, y - 1, z)

        # Destroyed spring. Add neighbors and below to blocks to update.
        del self.springs[x, z]

        self.new.update(neighbors)
        self.new.add(below)

    def process(self):
        w = factory.world

        for x, y, z in self.tracked:
            # Try each block separately. If it can't be done, it'll be
            # discarded from the set simply by not being added to the new set
            # for the next iteration.
            try:
                block = w.sync_get_block((x, y, z))
                if block == self.sponge:
                    self.add_sponge(w, x, y, z)
                elif block == self.spring:
                    self.add_spring(w, x, y, z)
                elif block == self.fluid:
                    self.add_fluid(w, x, y, z)
                else:
                    # Hm, why would a pending block not be any of the things we
                    # care about? Maybe it used to be a spring or something?
                    if (x, z) in self.springs:
                        self.remove_spring(x, y, z)
                    elif (x, y, z) in self.sponges:
                        self.remove_sponge(x, y, z)
            except ChunkNotLoaded:
                pass

        # Flush affected chunks.
        to_flush = set()
        for x, y, z in chain(self.tracked, self.new):
            to_flush.add((x // 16, z // 16))
        for x, z in to_flush:
            d = factory.world.request_chunk(x, z)
            d.addCallback(factory.flush_chunk)

        self.tracked = self.new
        self.new = set()

        # Prune, and reschedule.
        self.schedule()

    @inlineCallbacks
    def dig_hook(self, chunk, x, y, z, block):
        """
        Check for neighboring water that might want to spread.

        Also check to see whether we are, for example, dug ice that has turned
        back into water.
        """

        x += chunk.x * 16
        z += chunk.z * 16

        # Check for sponges first, since they will mark the entirety of the
        # area.
        if block == self.sponge:
            for coords in product(
                xrange(x - 3, x + 4),
                xrange(max(y - 3, 0), min(y + 4, 128)),
                xrange(z - 3, z + 4),
                ):
                self.tracked.add(coords)

        else:
            for (dx, dy, dz) in (
                ( 0, 0,  0),
                ( 0, 0,  1),
                ( 0, 0, -1),
                ( 0, 1,  0),
                ( 1, 0,  0),
                (-1, 0,  0)):
                coords = x + dx, y + dy, z + dz
                test_block = yield factory.world.get_block(coords)
                if test_block in (self.spring, self.fluid):
                    self.tracked.add(coords)

        self.schedule()

    before = ("build",)
    after = tuple()

class Water(Fluid):

    spring = blocks["spring"].slot
    fluid = blocks["water"].slot
    levels = 7

    sponge = blocks["sponge"].slot

    whitespace = (blocks["air"].slot, blocks["snow"].slot)
    meltables = (blocks["ice"].slot,)

    step = 0.2

    name = "water"

class Lava(Fluid):

    spring = blocks["lava-spring"].slot
    fluid = blocks["lava"].slot
    levels = 3

    whitespace = (blocks["air"].slot, blocks["snow"].slot)
    meltables = (blocks["ice"].slot,)

    step = 0.5

    name = "lava"

class Redstone(object):

    implements(IAutomaton, IDigHook)

    step = 0.2

    blocks = (blocks["redstone-wire"].slot,)

    def __init__(self):
        self.tracked = set()

        self.loop = LoopingCall(self.process)

    def start(self):
        if not self.loop.running:
            self.loop.start(self.step)

    def stop(self):
        if self.loop.running:
            self.loop.stop()

    def schedule(self):
        if self.tracked:
            self.start()
        else:
            self.stop()

    @inlineCallbacks
    def update_wires(self, x, y, z, enabled):
        """
        Trace the wires starting at a certain point, and either enable or
        disable them.
        """

        level = 0xf if enabled else 0x0
        traveled = set()
        traveling = set([(x, y, z)])

        while traveling:
            # Visit nodes.
            for coords in traveling:
                metadata = yield factory.world.get_metadata(coords)
                if metadata != level:
                    factory.world.set_metadata(coords, level)
                    traveled.add(coords)

            # Rotate the nodes from last time into the old list, generate the
            # new list again, and then do a difference update to avoid
            # visiting previously touched nodes.
            nodes = [(
                (i - 1, j, k    ),
                (i + 1, j, k    ),
                (i,     j, k - 1),
                (i,     j, k + 1))
                for (i, j, k) in traveling]
            traveling.clear()
            for l in nodes:
                for coords in l:
                    block = yield factory.world.get_block(coords)
                    if block == blocks["redstone-wire"].slot:
                        traveling.add(coords)
            traveling.difference_update(traveled)

            if level:
                level -= 1

    def process(self):
        for factory, x, y, z in self.tracked:
            world = factory.world
            block = factory.world.get_block((x, y, z))
            neighbors = ((x - 1, y, z), (x + 1, y, z), (x, y, z - 1),
                (x, y, z + 1))

            if block == blocks["redstone-torch"].slot:
                # Turn on neighboring wires, as appropriate.
                for coords in neighbors:
                    if (world.get_block(coords) ==
                        blocks["redstone-wire"].slot):
                        self.update_wires(factory, coords[0], coords[1],
                            coords[2], True)

            if block == blocks["redstone-torch-off"].slot:
                # Turn off neighboring wires, as appropriate.
                for coords in neighbors:
                    if (world.get_block(coords) ==
                        blocks["redstone-wire"].slot):
                        self.update_wires(factory, coords[0], coords[1],
                            coords[2], False)

            elif block == blocks["redstone-wire"].slot:
                # Get wire status from neighbors.
                if any(world.get_block(coords) ==
                    blocks["redstone-torch"].slot
                    for coords in neighbors):
                    # We should probably be lit.
                    self.update_wires(factory, x, y, z, True)
                else:
                    # Find the strongest neighboring wire, and use that.
                    new_level = max(factory.world.get_metadata(coords)
                        for coords in neighbors
                        if factory.world.get_block(coords) ==
                            blocks["redstone-wire"].slot)
                    if new_level > 0x0:
                        new_level -= 1
                    world.set_metadata((x, y, z), new_level)

    @inlineCallbacks
    def feed(self, coords):

        self.tracked.add(coords)

        # Wire wants state updates from its neighbors.
        block = yield factory.world.get_block(coords)
        if block == blocks["redstone-wire"].slot:
            x, y, z = coords
            self.tracked.update(((x - 1, y, z), (x + 1, y, z), (x, y, z - 1),
                (x, y, z + 1)))

    scan = naive_scan

    def dig_hook(self, chunk, x, y, z, block):
        pass

    name = "redstone"

    before = ("build",)
    after = tuple()

water = Water()
lava = Lava()
redstone = Redstone()



import unittest

from bravo.utilities.spatial import Block2DSpatialDict, Block3DSpatialDict

class TestBlock2DSpatialDict(unittest.TestCase):

    def setUp(self):
        self.sd = Block2DSpatialDict()

    def test_trivial(self):
        pass

    def test_setitem(self):
        self.sd[1, 2] = "testing"
        self.assertTrue((1, 2) in self.sd.buckets[0, 0])
        self.assertTrue("testing" in self.sd.buckets[0, 0].values())

    def test_setitem_offset(self):
        self.sd[17, 33] = "testing"
        self.assertTrue((17, 33) in self.sd.buckets[1, 2])
        self.assertTrue("testing" in self.sd.buckets[1, 2].values())

    def test_setitem_float_keys(self):
        self.sd[1.1, 2.2] = "testing"
        self.assertTrue((1.1, 2.2) in self.sd.buckets[0, 0])
        self.assertTrue("testing" in self.sd.buckets[0, 0].values())

    def test_keys_contains_offset(self):
        """
        Make sure ``keys()`` works properly with offset keys.
        """

        self.sd[17, 33] = "testing"
        self.assertTrue((17, 33) in self.sd.keys())

    def test_contains_offset(self):
        """
        Make sure ``__contains__()`` works properly with offset keys.
        """

        self.sd[17, 33] = "testing"
        self.assertTrue((17, 33) in self.sd)

    def test_near(self):
        self.sd[1, 1] = "first"
        self.sd[2, 2] = "second"
        results = list(self.sd.itervaluesnear((3, 3), 2))
        self.assertTrue("first" not in results)
        self.assertTrue("second" in results)

    def test_near_boundary(self):
        self.sd[17, 17] = "testing"
        results = list(self.sd.itervaluesnear((15, 15), 4))
        self.assertTrue("testing" in results)

    def test_near_negative(self):
        self.sd[0, 0] = "first"
        results = list(self.sd.itervaluesnear((-8, 0), 8))
        self.assertTrue("first" in results)

class TestBlock3DSpatialDict(unittest.TestCase):

    def setUp(self):
        self.sd = Block3DSpatialDict()

    def test_trivial(self):
        pass

    def test_near(self):
        self.sd[1, 1, 1] = "first"
        self.sd[2, 2, 2] = "second"
        results = list(self.sd.itervaluesnear((3, 3, 3), 3))
        self.assertTrue("first" not in results)
        self.assertTrue("second" in results)

    def test_near_negative(self):
        self.sd[0, 64, 0] = "first"
        results = list(self.sd.itervaluesnear((-3, 61, -3), 9))
        self.assertTrue("first" in results)



# vim: set fileencoding=utf8 :

import unittest

from numpy import array
from numpy.testing import assert_array_equal

from bravo.utilities.bits import unpack_nibbles, pack_nibbles
from bravo.utilities.chat import sanitize_chat
from bravo.utilities.coords import split_coords, taxicab2, taxicab3
from bravo.utilities.temporal import split_time

class TestCoordHandling(unittest.TestCase):

    def test_split_coords(self):
        cases = {
            (0, 0): (0, 0, 0, 0),
            (1, 1): (0, 1, 0, 1),
            (16, 16): (1, 0, 1, 0),
            (-1, -1): (-1, 15, -1, 15),
            (-16, -16): (-1, 0, -1, 0),
        }
        for case in cases:
            self.assertEqual(split_coords(*case), cases[case])

    def test_taxicab2(self):
        cases = {
            (1, 2, 3, 4): 4,
            (1, 2, 1, 2): 0,
            (2, 1, 4, 3): 4,
        }
        for case in cases:
            self.assertEqual(taxicab2(*case), cases[case])

    def test_taxicab3(self):
        cases = {
            (1, 2, 1, 3, 4, 2): 5,
            (1, 2, 3, 1, 2, 3): 0,
            (2, 1, 2, 4, 3, 1): 5,
        }
        for case in cases:
            self.assertEqual(taxicab3(*case), cases[case])

class TestBitTwiddling(unittest.TestCase):

    def test_unpack_nibbles(self):
        assert_array_equal(unpack_nibbles("a"), [1, 6])
        assert_array_equal(unpack_nibbles("nibbles"),
            [14, 6, 9, 6, 2, 6, 2, 6, 12, 6, 5, 6, 3, 7])

    def test_pack_nibbles(self):
        self.assertEqual(pack_nibbles(array([1, 6])), "a")
        self.assertEqual(
            pack_nibbles(array([14, 6, 9, 6, 2, 6, 3, 7])),
            "nibs")

    def test_nibble_reflexivity(self):
        self.assertEqual("nibbles",
            pack_nibbles(
                array(unpack_nibbles("nibbles"))
            )
        )

class TestStringMunging(unittest.TestCase):

    def test_sanitize_chat_color_control_at_end(self):
        message = u"§0Test§f"
        sanitized = u"§0Test"
        self.assertEqual(sanitize_chat(message), sanitized)

class TestNumberMunching(unittest.TestCase):

    def test_split_time(self):
        # Sunrise.
        self.assertEqual(split_time(0), (6, 0))
        # Noon.
        self.assertEqual(split_time(6000), (12, 0))
        # Sunset.
        self.assertEqual(split_time(12000), (18, 0))
        # Midnight.
        self.assertEqual(split_time(18000), (0, 0))



import unittest
import shutil
import tempfile

from twisted.python.filepath import FilePath

import bravo.chunk
import bravo.plugins.serializers
from bravo.nbt import TAG_Compound, TAG_List, TAG_String
from bravo.nbt import TAG_Double, TAG_Byte, TAG_Short

class TestAlphaUtilities(unittest.TestCase):

    def test_names_for_chunk(self):
        self.assertEqual(bravo.plugins.serializers.names_for_chunk(-13, 44),
            ("1f", "18", "c.-d.18.dat"))
        self.assertEqual(bravo.plugins.serializers.names_for_chunk(-259, 266),
            ("1p", "a", "c.-77.7e.dat"))

    def test_base36(self):
        self.assertEqual(bravo.plugins.serializers.base36(0),
            "0")
        self.assertEqual(bravo.plugins.serializers.base36(-47),
            "-1b")
        self.assertEqual(bravo.plugins.serializers.base36(137),
            "3t")
        self.assertEqual(bravo.plugins.serializers.base36(24567),
            "iyf")

class TestBetaUtilities(unittest.TestCase):

    def test_name_for_region(self):
        """
        From RegionFile.java's comments.
        """

        self.assertEqual(bravo.plugins.serializers.name_for_region(30, -3),
            "r.0.-1.mcr")
        self.assertEqual(bravo.plugins.serializers.name_for_region(70, -30),
            "r.2.-1.mcr")

class TestAlphaSerializerInit(unittest.TestCase):

    def test_not_url(self):
        self.assertRaises(Exception, bravo.plugins.serializers.Alpha,
            "/i/am/not/a/url")

    def test_wrong_scheme(self):
        self.assertRaises(Exception, bravo.plugins.serializers.Alpha,
            "http://www.example.com/")

class TestAlphaSerializer(unittest.TestCase):

    def setUp(self):
        self.d = tempfile.mkdtemp()
        self.folder = FilePath(self.d)
        self.serializer = bravo.plugins.serializers.Alpha('file://' + self.folder.path)

    def tearDown(self):
        shutil.rmtree(self.d)

    def test_trivial(self):
        pass

    def test_load_entity_from_tag(self):
        tag = TAG_Compound()
        tag["Pos"] = TAG_List(type=TAG_Double)
        tag["Pos"].tags = [TAG_Double(10), TAG_Double(5), TAG_Double(-15)]
        tag["Rotation"] = TAG_List(type=TAG_Double)
        tag["Rotation"].tags = [TAG_Double(90), TAG_Double(0)]
        tag["OnGround"] = TAG_Byte(1)
        tag["id"] = TAG_String("Item")

        tag["Item"] = TAG_Compound()
        tag["Item"]["id"] = TAG_Short(3)
        tag["Item"]["Damage"] = TAG_Short(0)
        tag["Item"]["Count"] = TAG_Short(5)

        entity = self.serializer._load_entity_from_tag(tag)
        self.assertEqual(entity.location.x, 10)
        self.assertEqual(entity.location.yaw, 90)
        self.assertEqual(entity.location.grounded, True)
        self.assertEqual(entity.item[0], 3)

    def test_save_chunk_to_tag(self):
        chunk = bravo.chunk.Chunk(1, 2)
        tag = self.serializer._save_chunk_to_tag(chunk)
        self.assertTrue("xPos" in tag["Level"])
        self.assertTrue("zPos" in tag["Level"])
        self.assertEqual(tag["Level"]["xPos"].value, 1)
        self.assertEqual(tag["Level"]["zPos"].value, 2)

    def test_save_data(self):
        data = 'Foo\nbar'
        self.serializer.save_plugin_data('plugin1', data)
        self.assertTrue(self.folder.child('plugin1.dat').exists())
        with self.folder.child('plugin1.dat').open() as f:
            self.assertEqual(f.read(), data)

    def test_no_data_corruption(self):
        data = 'Foo\nbar'
        self.serializer.save_plugin_data('plugin1', data)
        self.assertEqual(self.serializer.load_plugin_data('plugin1'), data)



# vim: set fileencoding=utf8 :

from twisted.trial import unittest

from construct import Container
from construct import ArrayError, MappingError

import bravo.packets.beta
import bravo.packets.infini

class TestPacketDataStructures(unittest.TestCase):

    def test_named_packets_exist(self):
        for name, slot in bravo.packets.beta.packets_by_name.iteritems():
            self.assertTrue(slot in bravo.packets.beta.packets,
                    "%d is missing" % slot)

    def test_packet_names_exist(self):
        for slot in bravo.packets.beta.packets.iterkeys():
            self.assertTrue(slot in bravo.packets.beta.packets_by_name.values(),
                    "%d is missing" % slot)

    test_packet_names_exist.todo = "Missing a couple packet names still"

    def test_packet_names_match(self):
        for name, slot in bravo.packets.beta.packets_by_name.iteritems():
            self.assertEqual(name, bravo.packets.beta.packets[slot].name)

class TestPacketParsing(unittest.TestCase):

    def test_ping(self):
        packet = ""
        parsed = bravo.packets.beta.packets[0].parse(packet)
        self.assertTrue(parsed)

    def test_login_signed_seed(self):
        packet = """AAAACQAGAEMAbwByAGIAaQBu6SZCF/j8DlQA""".decode("base64")
        parsed = bravo.packets.beta.packets[1].parse(packet)
        self.assertEqual(parsed.protocol, 9)
        self.assertEqual(parsed.username, "Corbin")
        self.assertEqual(parsed.seed, -1646555943028388268)
        self.assertEqual(parsed.dimension, 0)

    def test_handshake(self):
        packet = "\x00\x01\x00a"
        parsed = bravo.packets.beta.packets[2].parse(packet)
        self.assertEqual(parsed.username, "a")

    def test_handshake_unicode(self):
        packet = "\x00\x01\x00\xa7"
        parsed = bravo.packets.beta.packets[2].parse(packet)
        self.assertEqual(parsed.username, u"§")

    def test_chat_color(self):
        packet = """
        ABMAPACnAGYATQByAFoAdQBuAHoApwBmAD4AIABBAGwAcgBpAHQAZQ==
        """.decode("base64")
        parsed = bravo.packets.beta.packets[3].parse(packet)
        self.assertEqual(parsed.message, u"<§fMrZunz§f> Alrite")

    def test_time(self):
        packet = "\x00\x00\x00\x00\x00\x00\x00\x2a"
        parsed = bravo.packets.beta.packets[4].parse(packet)
        self.assertEqual(parsed.timestamp, 42)

    def test_orientation(self):
        packet = "\x45\xc5\x66\x76\x42\x2d\xff\xfc\x01"
        parsed = bravo.packets.beta.packets[12].parse(packet)
        self.assertEqual(parsed.orientation.pitch, 43.49998474121094)
        self.assertEqual(parsed.orientation.rotation, 6316.8076171875)

    def test_location(self):
        packet = """
        P/AAAAAAAABAAAAAAAAAAEAIAAAAAAAAQBAAAAAAAABAoAAAQMAAAAE=
        """.decode("base64")
        parsed = bravo.packets.beta.packets[13].parse(packet)
        self.assertEqual(parsed.position.x, 1)
        self.assertEqual(parsed.position.y, 2)
        self.assertEqual(parsed.position.stance, 3)
        self.assertEqual(parsed.position.z, 4)
        self.assertEqual(parsed.orientation.rotation, 5)
        self.assertEqual(parsed.orientation.pitch, 6)
        self.assertEqual(parsed.grounded.grounded, 1)

    def test_digging(self):
        packet = "\x03\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01"
        parsed = bravo.packets.beta.packets[14].parse(packet)
        self.assertEqual(parsed.state, "broken")
        self.assertEqual(parsed.x, -1)
        self.assertEqual(parsed.y, 255)
        self.assertEqual(parsed.z, -1)
        self.assertEqual(parsed.face, "+y")

    def test_build(self):
        packet = "\x00\x00\x00\x19@\x00\x00\x00@\x05\x00\x04@\x00\x12"
        parsed = bravo.packets.beta.packets[15].parse(packet)
        self.assertEqual(parsed.x, 25)
        self.assertEqual(parsed.y, 64)
        self.assertEqual(parsed.z, 64)
        self.assertEqual(parsed.face, "+x")
        self.assertEqual(parsed.primary, 4)
        self.assertEqual(parsed.count, 64)
        self.assertEqual(parsed.secondary, 18)

    def test_build_bad_face(self):
        packet = "\x00\x00\x00\x19@\x00\x00\x00@\x06\x00\x04@\x12"
        self.assertRaises(MappingError, bravo.packets.beta.packets[15].parse,
            packet)

    def test_animate(self):
        packet = "\x00\x00\x00\x03\x01"
        parsed = bravo.packets.beta.packets[18].parse(packet)
        self.assertEqual(parsed.eid, 3)
        self.assertEqual(parsed.animation, "arm")

    def test_animate_bad_animation(self):
        packet = "\x00\x00\x00\x03\x05"
        self.assertRaises(MappingError, bravo.packets.beta.packets[18].parse,
            packet)

    def test_mob(self):
        packet = "AAAPb1z///4wAAAH5v//9ZP3AAAEfw==".decode("base64")
        parsed = bravo.packets.beta.packets[24].parse(packet)
        self.assertEqual(parsed.eid, 3951)
        self.assertEqual(parsed.type, "cow")
        self.assertEqual(parsed.x, -464)
        self.assertEqual(parsed.y, 2022)
        self.assertEqual(parsed.z, -2669)
        self.assertEqual(parsed.yaw, -9)
        self.assertEqual(parsed.pitch, 0)
        # Not testing metadata; those tests are completely separate.

    def test_mob_pig(self):
        packet = "AAih/lr///YwAAAIYAAB/hCzAAAAEAB/".decode("base64")
        parsed = bravo.packets.beta.packets[24].parse(packet)
        self.assertEqual(parsed.metadata, {0: ("byte", 0), 16: ("byte", 0)})

class TestPacketAssembly(unittest.TestCase):

    def test_ping(self):
        container = Container()
        assembled = bravo.packets.beta.packets[0].build(container)
        self.assertEqual(assembled, "")

    def test_login(self):
        container = Container(protocol=0, username="", seed=0,
            dimension=0)
        assembled = bravo.packets.beta.packets[1].build(container)
        self.assertEqual(assembled,
            "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00")

    def test_time(self):
        container = Container(timestamp=42)
        assembled = bravo.packets.beta.packets[4].build(container)
        self.assertEqual(assembled, "\x00\x00\x00\x00\x00\x00\x00\x2a")

    def test_build(self):
        container = Container(x=25, y=64, z=64, face="+x", primary=4,
            secondary=18, count=64)
        assembled = bravo.packets.beta.packets[15].build(container)
        self.assertEqual(assembled,
            "\x00\x00\x00\x19@\x00\x00\x00@\x05\x00\x04@\x00\x12")

    def test_build_bad_face(self):
        container = Container(x=25, y=64, z=64, face="+q", id=4, count=64,
            damage=18)
        self.assertRaises(MappingError, bravo.packets.beta.packets[15].build,
            container)

class TestPacketHelpers(unittest.TestCase):

    def test_make_packet(self):
        packet = bravo.packets.beta.make_packet("ping")
        self.assertEqual(packet, "\x00")

    def test_alphastring(self):
        s = "Just a test"
        parser = bravo.packets.beta.AlphaString("test")
        self.assertEqual(parser.build(s),
            """AAsASgB1AHMAdAAgAGEAIAB0AGUAcwB0""".decode("base64"))

    def test_metadata_empty(self):
        s = "\x7f"
        self.assertRaises(ArrayError, bravo.packets.beta.metadata.parse, s)

    def test_metadata_value(self):
        s = "\x00\x04\x7f"
        parsed = bravo.packets.beta.metadata.parse(s)
        self.assertEqual(parsed[0].value, 4)

    def test_metadata_trailing(self):
        s = "\x00\x04\x7f\x7f"
        parsed = bravo.packets.beta.metadata.parse(s)
        self.assertEqual(len(parsed), 1)

    def test_metadata_build(self):
        d = {0: bravo.packets.beta.Metadata(type='byte', value=0)}
        built = bravo.packets.beta.metadata.build(d)
        self.assertEqual(built, "\x00\x00\x7f")

    def test_metadata_build_tuple(self):
        """
        Tuples can be used instead of the ``Metadata`` named tuple.
        """

        d = {0: ("byte", 0)}
        built = bravo.packets.beta.metadata.build(d)
        self.assertEqual(built, "\x00\x00\x7f")

class TestPacketIntegration(unittest.TestCase):

    def test_location_round_trip(self):
        """
        Test whether we are affected by an older Construct bug.
        """

        packet = """
        DUAaAAAAAAAAQFDPXCkAAABAUGeuFIAAAEAeAAAAAAAAAAAAAAAAAAAA
        """.decode("base64")

        header, payload = bravo.packets.beta.parse_packets(packet)[0][0]
        self.assertEqual(header, 13)
        self.assertEqual(payload.position.x, 6.5)
        self.assertEqual(payload.position.y, 67.24000000953674)
        self.assertEqual(payload.position.stance, 65.62000000476837)
        self.assertEqual(payload.position.z, 7.5)
        self.assertEqual(payload.orientation.rotation, 0.0)
        self.assertEqual(payload.orientation.pitch, 0.0)
        self.assertEqual(payload.grounded.grounded, 0)
        reconstructed = bravo.packets.beta.make_packet("location", payload)
        self.assertEqual(packet, reconstructed)

class TestInfiniPacketParsing(unittest.TestCase):

    def test_ping(self):
        raw = "\x00\x01\x00\x00\x00\x06\x00\x10\x00\x4d\x3c\x7d\x7c"
        parsed = bravo.packets.infini.packets[0].parse(raw)
        self.assertEqual(parsed.header.identifier, 0x00)
        self.assertEqual(parsed.header.flags, 0x01)
        self.assertEqual(parsed.payload.uid, 16)
        self.assertEqual(parsed.payload.timestamp, 5061757)

    def test_disconnect(self):
        raw = "\xff\x00\x00\x00\x00\x19\x00\x17Invalid client version!"
        parsed = bravo.packets.infini.packets[255].parse(raw)
        self.assertEqual(parsed.header.identifier, 0xff)
        self.assertEqual(parsed.payload.explanation,
            "Invalid client version!")

class TestInfiniPacketStream(unittest.TestCase):

    def test_ping_stream(self):
        raw = "\x00\x01\x00\x00\x00\x06\x00\x10\x00\x4d\x3c\x7d\x7c"
        packets, leftovers = bravo.packets.infini.parse_packets(raw)



"""
Module for specifying types of errors which might occur internally.
"""

# Errors which can be raised by serializers in the course of doing things
# which serializers might normally do.

class SerializerException(Exception):
    """
    Something bad happened in a serializer.
    """

class SerializerReadException(SerializerException):
    """
    A serializer had issues reading data.
    """

class SerializerWriteException(SerializerException):
    """
    A serializer had issues writing data.
    """

# Errors from plugin loading.

class InvariantException(Exception):
    """
    Exception raised by failed invariant conditions.
    """

class PluginException(Exception):
    """
    Signal an error encountered during plugin handling.
    """

# Errors from NBT handling.

class MalformedFileError(Exception):
    """
    Exception raised on parse error.
    """

# Errors from bravo clients.

class BuildError(Exception):
    """
    Something went wrong with the build.
    """

# Errors from the world.

class ChunkNotLoaded(Exception):
    """
    The requested chunk is not currently loaded. If you need it, you will need
    to request it yourself.
    """



from struct import Struct, error as StructError
from gzip import GzipFile
from UserDict import DictMixin

from bravo.errors import MalformedFileError

TAG_END = 0
TAG_BYTE = 1
TAG_SHORT = 2
TAG_INT = 3
TAG_LONG = 4
TAG_FLOAT = 5
TAG_DOUBLE = 6
TAG_BYTE_ARRAY = 7
TAG_STRING = 8
TAG_LIST = 9
TAG_COMPOUND = 10

class TAG(object):
    """Each Tag needs to take a file-like object for reading and writing.
    The file object will be initialised by the calling code."""
    id = None

    def __init__(self, value=None, name=None):
        self.name = name
        self.value = value

    #Parsers and Generators
    def _parse_buffer(self, buffer):
        raise NotImplementedError(self.__class__.__name__)

    def _render_buffer(self, buffer, offset=None):
        raise NotImplementedError(self.__class__.__name__)

    #Printing and Formatting of tree
    def tag_info(self):
        return self.__class__.__name__ + \
               ('("%s")'%self.name if self.name else "") + \
               ": " + self.__repr__()

    def pretty_tree(self, indent=0):
        return ("\t"*indent) + self.tag_info()

class _TAG_Numeric(TAG):
    def __init__(self, value=None, name=None, buffer=None):
        super(_TAG_Numeric, self).__init__(value, name)
        self.size = self.fmt.size
        if buffer:
            self._parse_buffer(buffer)

    #Parsers and Generators
    def _parse_buffer(self, buffer, offset=None):
        self.value = self.fmt.unpack(buffer.read(self.size))[0]

    def _render_buffer(self, buffer, offset=None):
        buffer.write(self.fmt.pack(self.value))

    #Printing and Formatting of tree
    def __repr__(self):
        return str(self.value)

#== Value Tags ==#
class TAG_Byte(_TAG_Numeric):
    id = TAG_BYTE
    fmt = Struct(">b")

class TAG_Short(_TAG_Numeric):
    id = TAG_SHORT
    fmt = Struct(">h")

class TAG_Int(_TAG_Numeric):
    id = TAG_INT
    fmt = Struct(">i")

class TAG_Long(_TAG_Numeric):
    id = TAG_LONG
    fmt = Struct(">q")

class TAG_Float(_TAG_Numeric):
    id = TAG_FLOAT
    fmt = Struct(">f")

class TAG_Double(_TAG_Numeric):
    id = TAG_DOUBLE
    fmt = Struct(">d")

class TAG_Byte_Array(TAG):
    id = TAG_BYTE_ARRAY
    def __init__(self, buffer=None):
        super(TAG_Byte_Array, self).__init__()
        self.value = ''
        if buffer:
            self._parse_buffer(buffer)

    #Parsers and Generators
    def _parse_buffer(self, buffer, offset=None):
        length = TAG_Int(buffer=buffer)
        self.value = buffer.read(length.value)

    def _render_buffer(self, buffer, offset=None):
        length = TAG_Int(len(self.value))
        length._render_buffer(buffer, offset)
        buffer.write(self.value)

    #Printing and Formatting of tree
    def __repr__(self):
        return "[%i bytes]" % len(self.value)

class TAG_String(TAG):
    id = TAG_STRING
    def __init__(self, value=None, name=None, buffer=None):
        super(TAG_String, self).__init__(value, name)
        if buffer:
            self._parse_buffer(buffer)

    #Parsers and Generators
    def _parse_buffer(self, buffer, offset=None):
        length = TAG_Short(buffer=buffer)
        read = buffer.read(length.value)
        if len(read) != length.value:
            raise StructError()
        self.value = unicode(read, "utf-8")

    def _render_buffer(self, buffer, offset=None):
        save_val = self.value.encode("utf-8")
        length = TAG_Short(len(save_val))
        length._render_buffer(buffer, offset)
        buffer.write(save_val)

    #Printing and Formatting of tree
    def __repr__(self):
        return self.value

#== Collection Tags ==#
class TAG_List(TAG):
    id = TAG_LIST
    def __init__(self, type=None, value=None, name=None, buffer=None):
        super(TAG_List, self).__init__(value, name)
        if type:
            self.tagID = type.id
        else: self.tagID = None
        self.tags = []
        if buffer:
            self._parse_buffer(buffer)
        if not self.tagID:
            raise ValueError("No type specified for list")

    #Parsers and Generators
    def _parse_buffer(self, buffer, offset=None):
        self.tagID = TAG_Byte(buffer=buffer).value
        self.tags = []
        length = TAG_Int(buffer=buffer)
        for x in range(length.value):
            self.tags.append(TAGLIST[self.tagID](buffer=buffer))

    def _render_buffer(self, buffer, offset=None):
        TAG_Byte(self.tagID)._render_buffer(buffer, offset)
        length = TAG_Int(len(self.tags))
        length._render_buffer(buffer, offset)
        for i, tag in enumerate(self.tags):
            if tag.id != self.tagID:
                raise ValueError("List element %d(%s) has type %d != container type %d" %
                         (i, tag, tag.id, self.tagID))
            tag._render_buffer(buffer, offset)

    #Printing and Formatting of tree
    def __repr__(self):
        return "%i entries of type %s" % (len(self.tags), TAGLIST[self.tagID].__name__)

    def pretty_tree(self, indent=0):
        output = [super(TAG_List,self).pretty_tree(indent)]
        if len(self.tags):
            output.append(("\t"*indent) + "{")
            output.extend([tag.pretty_tree(indent+1) for tag in self.tags])
            output.append(("\t"*indent) + "}")
        return '\n'.join(output)

class TAG_Compound(TAG, DictMixin):
    id = TAG_COMPOUND
    def __init__(self, buffer=None):
        super(TAG_Compound, self).__init__()
        self.tags = []
        if buffer:
            self._parse_buffer(buffer)

    #Parsers and Generators
    def _parse_buffer(self, buffer, offset=None):
        while True:
            type = TAG_Byte(buffer=buffer)
            if type.value == TAG_END:
                #print "found tag_end"
                break
            else:
                name = TAG_String(buffer=buffer).value
                try:
                    #DEBUG print type, name
                    tag = TAGLIST[type.value](buffer=buffer)
                    tag.name = name
                    self.tags.append(tag)
                except KeyError:
                    raise ValueError("Unrecognised tag type")

    def _render_buffer(self, buffer, offset=None):
        for tag in self.tags:
            TAG_Byte(tag.id)._render_buffer(buffer, offset)
            TAG_String(tag.name)._render_buffer(buffer, offset)
            tag._render_buffer(buffer,offset)
        buffer.write('\x00') #write TAG_END

    # Dict compatibility.
    # DictMixin requires at least __getitem__, and for more functionality,
    # __setitem__, __delitem__, and keys.

    def __getitem__(self, key):
        if isinstance(key,int):
            return self.tags[key]
        elif isinstance(key, str):
            for tag in self.tags:
                if tag.name == key:
                    return tag
            else:
                raise KeyError("A tag with this name does not exist")
        else:
            raise ValueError("key needs to be either name of tag, or index of tag")

    def __setitem__(self, key, value):
        if isinstance(key, int):
            # Just try it. The proper error will be raised if it doesn't work.
            self.tags[key] = value
        elif isinstance(key, str):
            value.name = key
            for i, tag in enumerate(self.tags):
                if tag.name == key:
                    self.tags[i] = value
                    return
            self.tags.append(value)

    def __delitem__(self, key):
        if isinstance(key, int):
            self.tags = self.tags[:key] + self.tags[key:]
        elif isinstance(key, str):
            for i, tag in enumerate(self.tags):
                if tag.name == key:
                    self.tags = self.tags[:i] + self.tags[i:]
                    return
            raise KeyError("A tag with this name does not exist")
        else:
            raise ValueError("key needs to be either name of tag, or index of tag")

    def keys(self):
        return [tag.name for tag in self.tags]


    #Printing and Formatting of tree
    def __repr__(self):
        return '%i Entries' % len(self.tags)

    def pretty_tree(self, indent=0):
        output = [super(TAG_Compound,self).pretty_tree(indent)]
        if len(self.tags):
            output.append(("\t"*indent) + "{")
            output.extend([tag.pretty_tree(indent+1) for tag in self.tags])
            output.append(("\t"*indent) + "}")
        return '\n'.join(output)


TAGLIST = {TAG_BYTE:TAG_Byte, TAG_SHORT:TAG_Short, TAG_INT:TAG_Int, TAG_LONG:TAG_Long, TAG_FLOAT:TAG_Float, TAG_DOUBLE:TAG_Double, TAG_BYTE_ARRAY:TAG_Byte_Array, TAG_STRING:TAG_String, TAG_LIST:TAG_List, TAG_COMPOUND:TAG_Compound}

class NBTFile(TAG_Compound):
    """Represents an NBT file object"""

    def __init__(self, filename=None, mode=None, buffer=None, fileobj=None):
        super(NBTFile,self).__init__()
        self.__class__.__name__ = "TAG_Compound"
        self.filename = filename
        self.type = TAG_Byte(self.id)
        #make a file object
        if filename:
            self.file = GzipFile(filename, mode)
        elif buffer:
            self.file = buffer
        elif fileobj:
            self.file = GzipFile(fileobj=fileobj)
        else:
            self.file = None
        #parse the file given intitially
        if self.file:
            self.parse_file()
            if filename and 'close' in dir(self.file):
                self.file.close()
            self.file = None

    def parse_file(self, filename=None, buffer=None, fileobj=None):
        if filename:
            self.file = GzipFile(filename, 'rb')
        elif buffer:
            self.file = buffer
        elif fileobj:
            self.file = GzipFile(fileobj=fileobj)
        if self.file:
            try:
                type = TAG_Byte(buffer=self.file)
                if type.value == self.id:
                    name = TAG_String(buffer=self.file).value
                    self._parse_buffer(self.file)
                    self.name = name
                    self.file.close()
                else:
                    raise MalformedFileError("First record is not a Compound Tag")
            except StructError:
                raise MalformedFileError("Partial File Parse: file possibly truncated.")
        else: ValueError("need a file!")

    def write_file(self, filename=None, buffer=None, fileobj=None):
        if buffer:
            self.file = buffer
        elif filename:
            self.file = GzipFile(filename, "wb")
        elif fileobj:
            self.file = GzipFile(fileobj=fileobj)
        elif self.filename:
            self.file = GzipFile(self.filename, "wb")
        elif not self.file:
            raise ValueError("Need to specify either a filename or a file")
        #Render tree to file
        TAG_Byte(self.id)._render_buffer(self.file)
        TAG_String(self.name)._render_buffer(self.file)
        self._render_buffer(self.file)
        #make sure the file is complete
        if 'flush' in dir(self.file):
            self.file.flush()
        if filename and 'close' in dir(self.file):
            self.file.close()

# Useful utility functions for handling large NBT structures elegantly and
# Pythonically.

def unpack_nbt(tag):
    """
    Unpack an NBT tag into a native Python data structure.
    """

    if isinstance(tag, TAG_List):
        return [unpack_nbt(i) for i in tag.tags]
    elif isinstance(tag, TAG_Compound):
        return dict((i.name, unpack_nbt(i)) for i in tag.tags)
    else:
        return tag.value

def pack_nbt(s):
    """
    Pack a native Python data structure into an NBT tag. Only the following
    structures and types are supported:

     * int
     * float
     * str
     * unicode
     * dict

    Additionally, arbitrary iterables are supported.

    Packing is not lossless. In order to avoid data loss, TAG_Long and
    TAG_Double are preferred over the less precise numerical formats.

    Lists and tuples may become dicts on unpacking if they were not homogenous
    during packing, as a side-effect of NBT's format. Nothing can be done
    about this.

    Only strings are supported as keys for dicts and other mapping types. If
    your keys are not strings, they will be coerced. (Resistance is futile.)
    """

    if isinstance(s, int):
        return TAG_Long(s)
    elif isinstance(s, float):
        return TAG_Double(s)
    elif isinstance(s, (str, unicode)):
        return TAG_String(s)
    elif isinstance(s, dict):
        tag = TAG_Compound()
        for k, v in s:
            v = pack_nbt(v)
            v.name = str(k)
            tag.tags.append(v)
        return tag
    elif hasattr(s, "__iter__"):
        # We arrive at a slight quandry. NBT lists must be homogenous, unlike
        # Python lists. NBT compounds work, but require unique names for every
        # entry. On the plus side, this technique should work for arbitrary
        # iterables as well.
        tags = [pack_nbt(i) for i in s]
        t = type(tags[0])
        # If we're homogenous...
        if all(t == type(i) for i in tags):
            tag = TAG_List(type=t)
            tag.tags = tags
        else:
            tag = TAG_Compound()
            for i, item in enumerate(tags):
                item.name = str(i)
            tag.tags = tags
        return tag
    else:
        raise ValueError("Couldn't serialise type %s!" % type(s))



from random import choice

motds = """
Open-source!
%
Distribute!
%
Reverse-engineered!
%
The work of MAD!
%
Made in the USA!
%
Asynchronous!
%
Seasons!
%
Sponges!
%
Simplex noise!
%
MIT-licensed!
%
Unit-tested!
%
Documented!
%
Password login!
%
Fluid simulations!
%
Whoo, Bukkit!
%
Whoo, Mineflayer!
%
Whoo, Mineserver!
%
Whoo, craftd!
%
Can't be held by Bukkit!
%
The test that stumped them all!
%
Comfortably numb!
%
The hidden riddle!
%
We are all made of stars!
%
Out of beta and releasing on time!
%
Still alive!
%
"Pentasyllabic" is an autonym!
"""

motds = [i.strip() for i in motds.split("%")]

def get_motd():
    """
    Retrieve a random MOTD.
    """

    return choice(motds)
